///////////////////////////////////////////////////////////////////////////
//
// File: ToolButton.cpp
//
// Description: Helper class for tool (mode) buttons
//
// Copyright 2010 Autodesk, Inc. All rights reserved.
// Use of this software is subject to the terms of the Autodesk license
// agreement provided at the time of installation or download, or which
// otherwise accompanies this software in either electronic or hard copy
// form.
//
///////////////////////////////////////////////////////////////////////////
#include <iostream>
#include <fstream>
#include <string>

#include <xsi_utils.h>
#include "PieMenu.h"
#include "ToolButton.h"

#include <xsi_application.h>


// Build circle points
void PieMenu::constructCircle (double in_r, spoint in_center, vector<spoint> &out_points) {
		out_points.clear();
		int i = 0;
		const double kRadianIncr = 6.28f/m_buttonCount;
		for (i = 0; i < m_buttonCount; i++)
		{
			double angleRadians = kRadianIncr*i;
			spoint p; 
			p.x = LONG(in_r * cos(angleRadians) + in_center.x );
			p.y = LONG(in_r * sin(angleRadians) + in_center.y );
			out_points.push_back(p);
		}
}
//linear interpolation

void PieMenu::lerp(spoint& dest, const spoint& a, const spoint& b, const double t)
{
		dest.x = (LONG)(a.x + (double)(b.x-a.x)*t);
		dest.y = (LONG)(a.y + (double)(b.y-a.y)*t);
}

PieMenu::PieMenu()
:	m_buttonCount( 0 )
{
	
}

PieMenu::~PieMenu()
{
}

void PieMenu::Setup(double in_r,LONG in_count, CString in_folderpath, CString in_pluginPath, LONG in_selectionIndex   )
{
	m_buttonCount	= in_count;
	m_maxRadius		= in_r;
	m_folderpath	= in_folderpath;
	m_pluginPath	= in_pluginPath;
	m_selectedIndex = in_selectionIndex;
	AddButtons(  );
}

void PieMenu::Cleanup()
{
	for (int i=0;i<m_buttonCount;i++)
		m_buttons[i].Cleanup();
	m_buttons.clear();	
}


void PieMenu::Draw( ToolContext &in_ctxt )
{
	constructCircle(m_maxRadius,m_center,m_menuPoints);
	in_ctxt.BeginViewDraw();
		for (int i=0;i<m_menuPoints.size();i++)
		{	
			m_buttons[i].SetPosition(m_menuPoints[i].x,m_menuPoints[i].y);
			m_buttons[i].Draw(in_ctxt);
		}
	in_ctxt.EndViewDraw();
}

void PieMenu::AddButtons()
{
		CString slash = CUtils::Slash();

		CString texturePath,namelistPath=m_pluginPath;
#ifndef linux
		texturePath =  L".." + slash + L".." + slash; // On windows plugin is located two levels below...
		namelistPath += texturePath;
#endif
		texturePath += L"data" +slash+ m_folderpath + slash + L"textures" + slash;
		namelistPath += L"data" +slash+ m_folderpath + slash + m_folderpath + L".txt";
		

		for(int i=0;i<m_buttonCount;i++)
		{
			CString bPath = texturePath + m_folderpath + CString(i);
			m_buttons.push_back(ToolButton());
			m_buttons[i].Setup( bPath.GetWideString(), i);
		}

		string l_str;
		ifstream infile;
		
		infile.open (namelistPath.GetWideString());
		
		int k=0;
        while(!infile.eof() && k < m_buttonCount) // To get you all the lines.
        {
	        std::getline(infile,l_str); // Saves the line in STRING.
			m_buttons[k].SetName(CString(l_str.data()));
	        k++;
        }
		infile.close();
}
bool PieMenu::SetSelection(ToolContext &in_ctxt, LONG mousex,LONG mousey)
{
	bool selected = false;
	CleanButtonSelectionData();
	for(int i=0;i<m_buttonCount;i++)
	{
		if(m_buttons[i].IsPointInside(in_ctxt, mousex,mousey)) 
		{
			m_selectedIndex = i;
			m_buttons[i].SetSelected(true);
			selected = true;
			break;
		}	
	}
	return selected;
}
void PieMenu::CleanButtonSelectionData()
{
	for(int i=0;i<m_buttonCount;i++)
		m_buttons[i].SetSelected(false);
}