// pmQuickPrimitive Plugin

#pragma warning (disable:4800)
#pragma warning (disable:4305)
#define GLEW_STATIC
#include "includes.h"

#define MOUSE_MOVE_TIME_DELAY 3

//#undef _DEBUG
//#define TEST_OGL

SICALLBACK XSILoadPlugin( PluginRegistrar& in_reg )
{

	in_reg.PutAuthor(L"piotrek marczak");
	in_reg.PutName(L"pmQuickPrimitive Plugin");
	in_reg.PutVersion(0,93);
	in_reg.RegisterTool(L"pmQuickPrimitive");	
	in_reg.RegisterMenu(siMenuTbGetPrimitiveID,L"pmQuickPrimitive_Menu",false,false);
	//RegistrationInsertionPoint - do not remove this line
	
	return CStatus::OK;
}

SICALLBACK XSIUnloadPlugin( const PluginRegistrar& in_reg )
{

	return CStatus::OK;
}

////////////////////////////////////////////////////////////////////////////////
//
// pmQuickPrimitive
//
////////////////////////////////////////////////////////////////////////////////

class pmQuickPrimitive {
private: // Data
	CString	m_pluginPath;
	// ---------------------- mouse data
	PieMenu	m_piemenu;
	PieMenu m_piemenuLights;
	bool	m_arealight; // set on mouse down, if true then rmb = change area of light
	bool	m_dirlight;// set on mouse down, if true then light is not rigged (null+lookat constrain) and directional
	CVector3	m_lightTarget;	//for m_dirlight, target position for unrigged, directional lights
	enum enMenuSel {kPrimMenu, kLightPrimMenu} m_MenuSel, m_lastMenuSel;
	enum enLastPrimType {kMesh, kLight} m_lastPrimType;
	LONG m_lastMenuInd;
	LONG m_mouseDownX,m_mouseDownY;
	LONG m_mouseDragX,m_mouseDragY;
	LONG m_mouseX,m_mouseY;
	POINT lp_mouse; //absolute mouse position
	CStatus m_lastMouseMoveStatus;
	// ----------------------

	enum enToolMode {kMenu,kPrimDrawing,kButtonDrag, kIdle} m_toolMode;
	enum enDragMode {kFirst,kSecond} m_dragMode;
	enum enAlignMode {kNormalAlign,kFrameAlign} m_alignMode;	//TODO normalAlign
	enum enDrawMode	{kCenterOut=0, kPointToPoint=1} m_drawMode;
	enum enToolScope {kGrid,kSelection,kScene} m_toolScope;	//work on grid/selection/scene
	
	// ---------------- picked data
	enum enSubcomponentType{ kVertex,kEdge, kPoly, kSurface, kNone }  m_pickedSubcompType;
	CString		m_pickedSubcompString;
	X3DObject	m_picked;

	Edge		m_pickedEdge;
	PolygonFace m_pickedPolyFace;
	Vertex		m_pickedVertex;
	CVector3	m_pickedSubcompPos, //position of subcomponent
				m_pickedNormal;		//normal of subcomponent
	CVector3	m_pickedFrameAxisX,m_pickedFrameAxisY,m_pickedFrameAxisZ,
				m_pickedCurveFrameAxisX,m_pickedCurveFrameAxisY,m_pickedCurveFrameAxisZ;	//temporary, for display purposes
	LONG		m_activeAxis;		//pick axis during mouse move
	bool		m_pickIsValid;		//valid position for picking
	// ----------------------
	// ------------------- Light prim data
	Light		m_light;
	LightRig	m_lightRig;
	CPlane		m_manipPlane, m_gridPlane;		// Manipulation plane
	CVector3	m_vertAxis;			// Vertical axis
	CVector3	m_intPos;			// Interest position
	CVector3	m_lightPos;			// Light position
	CValue		m_conevis;			// saved cone visibility
	// ------------------ created primitive data
	X3DObject	m_primitive;
	X3DObject	m_lastprimitive, m_lastLightprimitive;	//store last drawn primitive
	CString		m_primitiveType,
				m_lastprimitiveType;
	CValueArray m_lastprimitiveData;	
	Parameter	m_parameter[3]; //parameters to update				
	Parameter	m_parameterSec[3];//parameters to update after mouse up event (height of cylinder for example)		
	enum		enPrimDrawAxisSec { kPlane,kNormal } m_parameterDrawingAxisSec; //secondary axis when drawing primitive, first axis is always local reference frame
	int			m_paramCnt, m_paramSecCnt;
	double		m_heightFixRatio;
	CVector3	m_primitivePos;

	CRefArray	m_cprimitives; //ref array of created primitives /for merge/
	// ----------------------

	clock_t		m_tooltime;

	//preferences
	double		m_snapIncrement;
	int			m_planeScale;
	int			m_axisScale;

	//buttons
	ButtonBar	m_alignButtons;
	ButtonBar	m_axisButtons;
	ButtonBar	m_toolscopeButtons;
	ButtonBar	m_additionalButtons;

	CVector3	m_dumpvector[4];
public: // Methods
	pmQuickPrimitive()
	{}

	~pmQuickPrimitive()
	{}

	CStatus Setup( ToolContext& in_ctxt )
	{
		Application().LogMessage( L"pmQuickPrimitive started" );
		Plugin thisPlugin = Application().GetPlugins().GetItem( L"pmQuickPrimitive Plugin" ) ;
		m_pluginPath = thisPlugin.GetOriginPath() ;

		ClearAllData();

		if( GLEW_OK != glewInit() ){
			logmsg("Couldn't initialize GLEW");
			return CStatus::Abort;
		}
		// **********************
		// setup primitive menu
		// **********************
		
		// **********************
		// setup lights menu
		/*
			Renderer names
			Arnold Render
			Hardware Renderer
			3delight
			mental ray
			Maxwell
			VRay
		*/
		// **********************
		CString rendererName = Application().GetActiveProject().GetActiveScene().GetActivePass().GetRenderer().GetName(); 
		if ( rendererName == L"VRay")	{
			m_piemenu.Setup(105,10,L"piemenu", m_pluginPath,0);
			m_piemenuLights.Setup(64, 8, "piemenuLightsVray", m_pluginPath, 0);
		}
		else
		{
			m_piemenu.Setup(80,10,L"piemenu", m_pluginPath,0);
			m_piemenuLights.Setup(40, 5, "piemenuLights", m_pluginPath, 0);
		}
			
		// **********************
		// INIT button data
		// **********************
		CStringArray l_stra; 
		l_stra.Add(L"pointAlign");l_stra.Add(L"edgeAlign");l_stra.Add(L"polyAlign");
		m_alignButtons.Setup(0,10,L"buttons",m_pluginPath,false,l_stra);		
		l_stra.Clear();l_stra.Add(L"XAlign");l_stra.Add(L"YAlign");l_stra.Add(L"ZAlign");
		m_axisButtons.Setup(3,10,L"buttons",m_pluginPath,true,l_stra);		
		l_stra.Clear();l_stra.Add(L"gridScope");l_stra.Add(L"selectedScope");l_stra.Add(L"sceneScope");	
		m_toolscopeButtons.Setup(6,10,L"buttons",m_pluginPath,true,l_stra);
		l_stra.Clear();l_stra.Add(L"merge");	
		m_additionalButtons.Setup(7,10,L"buttons",m_pluginPath,false,l_stra,false);		
		m_additionalButtons.SetSelected(0,false);
		// **********************
		//	Load GLOBAL saved data
		// **********************
		LONG bd1=0,bd2=0,bd3=0;
		if ( GetSavedData(L"pmQuickPrimitive_ptAlignButtonSelected", bd1 ) != CStatus::OK ) bd1 = 1;
		if ( GetSavedData(L"pmQuickPrimitive_edgeAlignButtonSelected",bd2)  != CStatus::OK ) bd2 = 1;
		if ( GetSavedData(L"pmQuickPrimitive_polyAlignButtonSelected",bd3) != CStatus::OK ) bd3 = 1;
		m_alignButtons.SetSelected(0,(bool)bd1);
		m_alignButtons.SetSelected(1,(bool)bd2);
		m_alignButtons.SetSelected(2,(bool)bd3);
		// INIT XYZ button data
		LONG xyzsaved;
		if ( GetSavedData(L"pmQuickPrimitive_XYZAlignButtonSelected",xyzsaved) == CStatus:: OK)
			m_axisButtons.SetSelection(xyzsaved);
		else
			m_axisButtons.SetSelection(1);
		if ( GetSavedData(L"pmQuickPrimitive_ToolScopeButtonSelected",xyzsaved) == CStatus:: OK)
			m_toolscopeButtons.SetSelection(xyzsaved);
		else
		{
			LONG outv;
			GetFromPrefs( L"ToolScope", outv);
			m_toolscopeButtons.SetSelection(outv);
		}
			
		m_alignMode = kFrameAlign;

		//**************************************************************
		// PREFRENCES
#ifdef _DEBUG
		logmsg("loading prefs");
#endif
		CValue ipref;
		CString prefname = L"QuickPrimitivePreferences";

		if(Application().GetPreferences().GetPreferenceValue(prefname+L".CubeV", ipref) != CStatus::OK)	{		
			CValue outArg;
			CValueArray inArgs; 
			Application().ExecuteCommand(L"RefreshCustomPreferences",inArgs,outArg);
			logmsg("Installing quickPrimitive preferences...");
		}
		LONG but_sel = 0;
		LONG menu_sel = 0;
		if ( GetSavedData(L"pmQuickPrimitive_piemenuInd", menu_sel) != CStatus::OK )	//which menu is selected
		{
			menu_sel = 0;
		}
		switch (menu_sel)	{
			case 0: m_MenuSel = kPrimMenu;break;
			case 1: m_MenuSel = kLightPrimMenu;break;
		};
		if (menu_sel == 0 )	{
			if ( GetSavedData(L"pmQuickPrimitive_piemenuselectionIndex", but_sel) != CStatus::OK )
			{
				int t;
				if ( GetFromPrefs("DefaultPrimitive", t) != CStatus::OK)
					but_sel = 0;
				else 
					but_sel = t;
			}
			m_piemenu.SetSelection(but_sel);
		}
		else if (menu_sel == 1 )	{
			if ( GetSavedData(L"pmQuickPrimitive_piemenuLightsselectionIndex", but_sel) != CStatus::OK )
			{
				int t;
				if ( GetFromPrefs("DefaultPrimitive", t) != CStatus::OK)
					but_sel = 0;
				else 
					but_sel = t;
			}
			m_piemenuLights.SetSelection(but_sel);
		}
		CValue ret;
		Application().GetPreferences().GetPreferenceValue(L"TransformProperties.scalesnap",ret);
		m_snapIncrement = ret;
		if ( GetFromPrefs(L"PlaneHelperScale", m_planeScale) != CStatus::OK)
			m_planeScale = 50;
		if ( GetFromPrefs(L"AxisHelperScale", m_axisScale) != CStatus::OK)
			m_axisScale = 45;
		
		//**************************************************************
		
		return CStatus::OK;
	}

	CStatus GetFromPrefs(CString in_str, LONG &out_val)
	{
		CValue ret;
		CStatus st =  Application().GetPreferences().GetPreferenceValue(L"QuickPrimitivePreferences."+in_str, ret);
		out_val = ret;
		return st;
	}
	LONG GetFromPrefs( CString in_str )
	{
		CValue retv;
		CStatus st =  Application().GetPreferences().GetPreferenceValue(L"QuickPrimitivePreferences."+in_str, retv);
		return st == CStatus::OK ? (LONG)retv : UINT_MAX;
	}
	bool GetFromPrefs2( CString in_str )
	{
		CValue retv;
		CStatus st =  Application().GetPreferences().GetPreferenceValue(L"QuickPrimitivePreferences."+in_str, retv);
		return st == CStatus::OK ? (bool)retv : UINT_MAX;
	}
	CStatus GetFromPrefs(CString in_str, int &out_val)
	{
		CValue ret;
		CStatus st =  Application().GetPreferences().GetPreferenceValue(L"QuickPrimitivePreferences."+in_str, ret);
		out_val = ret;
		return st;
	}
	CStatus GetFromPrefs(CString in_str, double &out_val)
	{
		CValue ret;
		CStatus st =  Application().GetPreferences().GetPreferenceValue(L"QuickPrimitivePreferences."+in_str, ret);
		ret.ChangeType(CValue::siFloat);
		out_val = ret;
		return st;
	}
	CStatus GetFromPrefs(CString in_str, CValue &out_val)
	{
		CValue ret;
		CStatus st =  Application().GetPreferences().GetPreferenceValue(L"QuickPrimitivePreferences."+in_str, ret);
		out_val = ret;
		return st;
	}
	CStatus Cleanup( ToolContext& in_ctxt )
	{
#ifdef _DEBUG
		logmsg("* Performing CLEANUP");
#endif
		SaveData(m_MenuSel,L"pmQuickPrimitive_piemenuInd");
		SaveData(m_piemenu.GetSelectionIndex(),L"pmQuickPrimitive_piemenuselectionIndex");
		SaveData(m_piemenuLights.GetSelectionIndex(),L"pmQuickPrimitive_piemenuLightsselectionIndex");
		SaveData(m_alignButtons.IsSelected(0),L"pmQuickPrimitive_ptAlignButtonSelected");
		SaveData(m_alignButtons.IsSelected(1),L"pmQuickPrimitive_edgeAlignButtonSelected");
		SaveData(m_alignButtons.IsSelected(2),L"pmQuickPrimitive_polyAlignButtonSelected");

		SaveData(m_axisButtons.GetSelectionIndex(),L"pmQuickPrimitive_XYZAlignButtonSelected");
		SaveData(m_toolscopeButtons.GetSelectionIndex(),L"pmQuickPrimitive_ToolScopeButtonSelected");
		
		m_piemenu.Cleanup();	
		m_piemenuLights.Cleanup();
		m_alignButtons.Cleanup();
		m_axisButtons.Cleanup();
		m_toolscopeButtons.Cleanup();
		m_additionalButtons.Cleanup();
		ClearAllData();

		LogOpenGLErrors();
		return CStatus::OK;
	}

	CStatus Activate( ToolContext& in_ctxt )
	{
		// Called when tool becomes the active tool
		in_ctxt.SetCursor( siCrossCursor );

		m_tooltime = clock();
		in_ctxt.EnableSnapping( true );

		//Uncomment this to setup a custom tool shortcut key
		in_ctxt.RegisterShortcutKey( kSubdiv1, L"Subdivide Prim A" );
		in_ctxt.RegisterShortcutKey( kSubdiv2, L"Subdivide Prim B" );
		in_ctxt.RegisterShortcutKey( kSubdiv3, L"Subdivide Prim C" );
	
		in_ctxt.RegisterShortcutKey( kSubdiv1, L"Subdivide Down Prim  A",siShiftMask );
		in_ctxt.RegisterShortcutKey( kSubdiv2, L"Subdivide Down Prim  B",siShiftMask );
		in_ctxt.RegisterShortcutKey( kSubdiv3, L"Subdivide Down Prim  C",siShiftMask );
		in_ctxt.RegisterShortcutKey( 0x0D, L"Inspect last operator" );
		in_ctxt.SetToolDescription( L"pmQuickPrimitive\nCreate Primitive\nDuplicate Last\nMenu" );
		CValue ret;
		Application().GetPreferences().GetPreferenceValue(L"TransformProperties.scalesnap",ret);

		m_snapIncrement = ret;
		GetFromPrefs(L"PlaneHelperScale", m_planeScale);
		GetFromPrefs(L"AxisHelperScale", m_axisScale);

		m_pickIsValid = false;
		return CStatus::OK;
	}

	CStatus Deactivate( ToolContext& in_ctxt )
	{
	
		// Called when tool becomes inactive (e.g. when user orbits using nav or chooses another tool)
		return CStatus::OK;
	}
	//************************************************************************************************************************** Mouse MOVE
	CStatus MouseDown( ToolContext& in_ctxt )
	{
		// Called when mouse button is pressed
		LONG x, y;
		in_ctxt.GetMousePosition( x, y );
		m_mouseDownX = x;
		m_mouseDownY = y;
		m_mouseDragX = x;
		m_mouseDragY = y;

		m_toolMode = kIdle;
		if ( in_ctxt.IsLeftButtonDown() )	
		{		
			// check for buttons
			LONG l_index;
			if( m_alignButtons.IsPointInside(in_ctxt,x,y,l_index) )	{
				m_alignButtons.SetSelected(l_index,!m_alignButtons.IsSelected(l_index) );
				return CStatus::Abort;
			}
			if (m_axisButtons.IsPointInside(in_ctxt,x,y,l_index) )	{
				m_axisButtons.SetSelection(l_index);
				return CStatus::Abort;
			}
			if (m_toolscopeButtons.IsPointInside(in_ctxt,x,y,l_index) )	{
				m_toolscopeButtons.SetSelection(l_index);
				return CStatus::Abort;
			}
			if( m_additionalButtons.IsPointInside(in_ctxt,x,y,l_index) )	{
				m_additionalButtons.SetSelected(l_index, true);
				m_toolMode = kButtonDrag;
				return CStatus::OK;
			}
			//	get selection under cursor
			CVector3 l_pos;
			CStatus l_snapStatus = GetSnapData(in_ctxt, x,y,l_pos);
			if (l_snapStatus != CStatus::OK)
			{
				m_pickIsValid = false;
				ClearPickedData();
				return CStatus::False;
			}
			m_pickIsValid = true;
			m_primitivePos = l_pos;
			m_toolMode = kPrimDrawing;
			
			if (m_MenuSel == kPrimMenu)
			{
				// if shift is pressed when LMB, grid/cube primitive center will be from mouse down point
				int l_p;
				if( (m_piemenu.GetSelectionName() == "Grid" || m_piemenu.GetSelectionName() == "Cube") )
					GetFromPrefs("DrawModeBox",l_p);
				else
					GetFromPrefs("DrawModeSphere",l_p);

				if(in_ctxt.IsShiftKeyDown()){
					if (l_p == 0)
						m_drawMode = kPointToPoint;
					else
						m_drawMode = kCenterOut;
				}
				else	{
					if (l_p == 0)
						m_drawMode = kCenterOut;
					else
						m_drawMode = kPointToPoint;
				}		
				if ( CreatePrimitive(in_ctxt, m_piemenu.GetSelectionName(), m_primitivePos, m_primitive) != CStatus::OK)	//primitive was not created
				{
					ClearPickedData();
					m_toolMode = kIdle;
					return CStatus::False;
				}
				BeginParamUpdate(in_ctxt, m_primitive);		
			}
			else 
			if ( m_MenuSel == kLightPrimMenu )	
			{
				CPlane l_ViewPlane;
				in_ctxt.GetGridPlane( m_gridPlane );
				in_ctxt.GetViewPlane( l_ViewPlane );
				m_manipPlane = m_gridPlane;
				in_ctxt.SetManipulationPlane( m_manipPlane );
				if ( !m_manipPlane.Parallel( l_ViewPlane ) )
					m_vertAxis = m_manipPlane.GetNormal();
				else
					m_vertAxis.PutNull(); // Normal is orthogonal to view

				m_lightPos = m_intPos = l_pos;
				// create lights
				CString rendererName = Application().GetActiveProject().GetActiveScene().GetActivePass().GetRenderer().GetName(); 
				Model oRoot = Application().GetActiveSceneRoot();
				m_light = Light();
				m_lightRig = LightRig();
				CString selName = m_piemenuLights.GetSelectionName();
				CString presetName = L"", lightName = L"";
				bool	lightRig = false;
				
				if (selName == L"Neon" || selName == L"Light_box" || selName == L"Spot" 
					|| selName == L"VRay Sun" || selName == L"VRay IES")
					lightRig = true;	
				if ( selName.FindString(L"VRay") == UINT_MAX )
					presetName = lightName = selName;
				else
				{
					// for vray lights use inifinte preset
					lightName = selName;
					presetName = L"Infinite";
				}
				
				if (selName == L"Neon" || selName == L"Light_box" || selName == L"SpotDisk" || selName == L"VRay Area" || selName == L"VRay Sphere Area" || selName == L"VRay Dome")
					m_arealight = true;
				else
					m_arealight = false;
				
				if ( (selName == L"Infinite" || selName == L"VRay Area" || selName == L"VRay Sphere Area") && lightRig == false)
					m_dirlight = true;
				else
					m_dirlight = false;
				CStatus st;
				if (lightRig)	
					st = oRoot.AddLightRig(presetName, lightName, m_lightRig );
				else
					st = oRoot.AddLight(presetName, false, lightName, m_light );
				if (st != CStatus::OK)	{
					ClearPickedData();
					m_toolMode = kIdle;
					return CStatus::False;
				}
				Light l;
				l = m_lightRig.IsValid() ? m_lightRig.GetLight() : m_light;
				SelectObject( l,false );
				// ***************
				// Set some Vray specific options
				// ***************
				if( (rendererName == L"VRay") && selName.FindString(L"VRay") != UINT_MAX)	
				{
					bool area = selName == L"VRay Area" || selName == L"VRay Sphere Area" || selName == L"VRay Dome";
					LONG areageom = selName == L"VRay Sphere Area" ? 3 : selName == L"VRay Area" ? 1 : 2;
					if (area){
						l.GetActivePrimitive().PutParameterValue(L"LightArea",true);			
						l.GetActivePrimitive().PutParameterValue(L"LightAreaGeom", areageom);
						double ax = 4.0, ay = 4.0, az = 1.0;
						if (selName == L"VRay Dome")
							ax = 100.0;
						l.GetActivePrimitive().PutParameterValue(L"LightAreaXformSX",ax);
						l.GetActivePrimitive().PutParameterValue(L"LightAreaXformSY",ay);
					}
					CString lightshader = selName == L"VRay IES" ? L"VRayLightIES" : L"VRayLight";
					Shader out_shader = ApplyShader(L"VRay",l.GetActivePrimitive(), L"Light", lightshader, L"LightShader");
					if (out_shader.IsValid())
						out_shader.PutParameterValue(L"m_uType", areageom-1);
				}
				// ***************
				// force show cones
				// ***************
				CValue lval;
				GetFromPrefs(L"Showlightscone", lval );
				if ( (bool)lval )
				{
					CValue result;
					CValueArray args;
					args.Add( CValue( -1 ) );
					if ( Application().ExecuteCommand( L"GetViewCamera", args, result ) == CStatus::OK ) {
						Camera l_cam = result;
						Property p;	l_cam.GetPropertyFromName(L"camvis",p);		
						m_conevis = p.GetParameterValue(L"attrselcones" );
						p.PutParameterValue(L"attrselcones", (bool)true );
					}
					else logmsg("Getting camera fail!");

				}
			
				// Begin updating parameters
				if ( lightRig )	{
					KinematicState interestKine = l.GetInterest().GetKinematics().GetGlobal();
					interestKine.GetParameter(L"posx").PutValue(l_pos.GetX());
					interestKine.GetParameter(L"posy").PutValue(l_pos.GetY());
					interestKine.GetParameter(L"posz").PutValue(l_pos.GetZ());
				}
				else
					{
						m_lightTarget = l_pos;
					}
				// **********************
				// Begin updating light parameters
				// **********************
				KinematicState lightKine = l.GetKinematics().GetGlobal();
				Parameter posX = lightKine.GetParameter(L"posx");
				Parameter posY = lightKine.GetParameter(L"posy");
				Parameter posZ = lightKine.GetParameter(L"posz");
				if ( selName == L"VRay Dome" )	{
					lightKine.PutParameterValue(L"rotx", (double)-90 );
				}
				in_ctxt.BeginParameterUpdate( posX );
				in_ctxt.BeginParameterUpdate( posY );
				in_ctxt.BeginParameterUpdate( posZ );
				in_ctxt.UpdateParameter( posX, l_pos.GetX() );
				in_ctxt.UpdateParameter( posY, l_pos.GetY() );
				in_ctxt.UpdateParameter( posZ, l_pos.GetZ() );
				in_ctxt.BeginParameterUpdate( lightKine.GetParameter(L"rotx") );
				in_ctxt.BeginParameterUpdate( lightKine.GetParameter(L"roty") );
				in_ctxt.BeginParameterUpdate( lightKine.GetParameter(L"rotz") );
				if (m_arealight)	{				
					in_ctxt.BeginParameterUpdate( l.GetActivePrimitive().GetParameter(L"LightAreaXformSX") );
					in_ctxt.BeginParameterUpdate( l.GetActivePrimitive().GetParameter(L"LightAreaXformSY") );
					in_ctxt.BeginParameterUpdate( l.GetActivePrimitive().GetParameter(L"LightAreaXformSZ") );
				}
				// **********************
				// Turn on shadows
				// **********************
				GetFromPrefs(L"LightShadow", lval );
				
				if ( (bool)lval  )	{
					CRefArray lshad = l.FindShaders(siShaderFilter);								
					for (LONG i=0; i < lshad.GetCount(); i++)
					{
						Shader shad = lshad[ i ];					
						if (shad.GetName().FindString(L"soft_light") != UINT_MAX )	{
							shad.PutParameterValue(L"shadow", true);
							shad.PutParameterValue(L"factor", (double)0.0 );
						}
					}
				}
				// **********************
			}
			in_ctxt.SetCursor(siDragXYCursor);
			GetCursorPos(&lp_mouse);
			return CStatus::OK;
		}
		else 
			if ( in_ctxt.IsRightButtonDown() )	{
				m_piemenu.SetPosition(x,y);
				m_piemenuLights.SetPosition(x,y);
				m_toolMode = kMenu;
				m_lastMenuSel = m_MenuSel;
				if (m_lastMenuSel == kLightPrimMenu )
					m_lastMenuInd = m_piemenuLights.GetSelectionIndex();
				else
					m_lastMenuInd = m_piemenu.GetSelectionIndex();
				in_ctxt.SetCursor(siDotCursor);
				return CStatus::OK;
			}
			else
				if ( in_ctxt.IsMiddleButtonDown() )		/*			Duplicate last primtive */
				{
					if  (!m_lastprimitive.IsValid() )
							return CStatus::False;
					CVector3 l_pos;
					CStatus l_snapStatus = GetSnapData(in_ctxt, x,y,l_pos);
					if (l_snapStatus != CStatus::OK)
					{
						ClearPickedData();
						return CStatus::False;
					}
					

					X3DObject l_primitive;
					
					if ( CreatePrimitive(in_ctxt, m_primitiveType,l_pos, l_primitive) != CStatus::OK)	//primitive was not created
					{
						ClearPickedData();
						m_toolMode = kIdle;
						return CStatus::False;
					}
					
					m_primitive = l_primitive;
					m_primitivePos = l_pos;

					double height = LoadPrimtiveData(m_primitiveType,m_primitive);

					CTransformation lTransform = m_primitive.GetKinematics().GetGlobal().GetTransform();	
					
					double fix_y;
					fix_y = 0.5;
		
					if (m_primitiveType == L"Sphere" || (m_primitiveType == L"Torus"))
						fix_y = 1;
					else
						if (m_primitiveType == L"Grid" || (m_primitiveType == L"Disc")	|| (m_primitiveType == L"Circle") || (m_primitiveType == L"Null"))
							fix_y=0;
						else
							if (m_primitiveType == L"Dodecahedron")
									fix_y = 0.79465;
							else 
								if (m_primitiveType == L"Cone")
										fix_y = 1.0/3.0;

					height *= fix_y;
					lTransform=lTransform.AddLocalTranslation(CVector3(0,height,0));
					m_primitive.GetKinematics().GetGlobal().PutTransform(lTransform);

					SavePrimtiveData(m_primitiveType, m_primitive);
					AddProjectionToPrimitive( in_ctxt,m_primitive,m_primitiveType );
					FreezeModellingStack( m_primitive );
					if ( m_primitiveType == L"Cube")
						FreezeCubeScale( m_primitive );
					m_cprimitives.Add(m_primitive);
					return CStatus::False; //ignore mouse up callback
				}
		return CStatus::False; // Ignore other mouse buttons etc.
	}
	//*********************************************************************************************************************** Mouse Drag
	CStatus MouseDrag( ToolContext& in_ctxt )
	{
		LONG x, y;
		in_ctxt.GetMousePosition( x, y );
		m_mouseDragX = x;
		m_mouseDragY = y;
		// Called when mouse is moved with a mouse button pressed

		if ( in_ctxt.IsLeftButtonDown() || in_ctxt.IsRightButtonDown() )
		{
			if (m_toolMode == kPrimDrawing )
			{
				if (m_MenuSel == kPrimMenu)
					return UpdatePrimitive(in_ctxt);
				else
					if(m_MenuSel == kLightPrimMenu )
						return UpdateLightPrimitive( in_ctxt );
			}
		}
		if (m_toolMode == kMenu && in_ctxt.IsRightButtonDown())
		{	
			if ( !m_piemenu.SetSelection(in_ctxt,x,y) )	
			{
				if ( m_piemenuLights.SetSelection(in_ctxt,x,y) )	{
					m_MenuSel = kLightPrimMenu;
					m_piemenu.SetSelectionNone();
				}
				else
				{
					m_MenuSel = m_lastMenuSel;
					if (m_lastMenuSel == kLightPrimMenu )	{
						m_piemenuLights.SetSelection(m_lastMenuInd);
						m_piemenu.SetSelectionNone();
					}
					else
					{
						m_piemenu.SetSelection(m_lastMenuInd);
						m_piemenuLights.SetSelectionNone();
					}
				}
				
			}
			else
			{
				m_piemenuLights.SetSelectionNone();
				m_MenuSel = kPrimMenu;
			}
				
		}
		return CStatus::OK;
	}
	//******************************************************************************************************************** Mouse Up
	CStatus MouseUp( ToolContext& in_ctxt )
	{
		// Called when mouse button is released
		LONG x, y;
		in_ctxt.GetMousePosition( x, y );
		
		if ( in_ctxt.IsLeftButtonDown() || in_ctxt.IsRightButtonDown() )
		{
			LONG l_ind = 0;
			if ( m_additionalButtons.IsPointInside(in_ctxt,x,y,l_ind) && m_toolMode == kButtonDrag )	{
				//merge 
				if (l_ind == 0) 	{
					X3DObject l_newobj;
					CStatus l_ms;
					CRefArray l_primlist;
					if ( !in_ctxt.IsShiftKeyDown() ) 
						l_primlist = m_cprimitives;
					else
						l_primlist.Add(m_primitive);
					l_ms = MergePrimitiveToSelected( in_ctxt,l_primlist,l_newobj );

					if (l_ms == CStatus::OK)
						ClearAllData();
				}				
			}
			else	// end primitive drawing, update paramteres for undo and save data
				if (m_toolMode == kPrimDrawing)
				{
					if (m_MenuSel == kPrimMenu)
					{
						EndParamUpdate(in_ctxt, m_primitive);
				
						if ( m_primitive.IsValid() )	{
							m_lastPrimType = kMesh;
							// Fix cube size/scaling issue
							if ( m_primitiveType == L"Cube")	{
						
								Parameter pscx = m_primitive.GetKinematics().GetGlobal().GetParameter(L"sclx");
								Parameter pscy = m_primitive.GetKinematics().GetGlobal().GetParameter(L"scly");
								Parameter pscz = m_primitive.GetKinematics().GetGlobal().GetParameter(L"sclz");
								Parameter psize = m_primitive.GetParameter(L"length");
								double maxscale;
								if (  GetFromPrefs(L"CubeCreation") == 0 )	
								{
									maxscale = (double)pscx.GetValue();
									if ( (double)pscy.GetValue() > maxscale )
										maxscale = pscy.GetValue();
									else
										if ( (double)pscz.GetValue() > maxscale )
											maxscale = pscz.GetValue();
								}
								else
									maxscale = 100.0;
								pscx.PutValue( (double)pscx.GetValue() / maxscale );
								pscy.PutValue( (double)pscy.GetValue() / maxscale );
								pscz.PutValue( (double)pscz.GetValue() / maxscale );
								psize.PutValue ( maxscale *0.01 );
								
							}
				
							m_lastprimitive = m_primitive;
							m_lastprimitiveType = m_primitiveType;
							SavePrimtiveData(m_primitiveType, m_primitive); //save data about last created primtive
							// add to list of created primitives
							m_cprimitives.Add(m_primitive);
							//  ADD PROJECTION
							AddProjectionToPrimitive( in_ctxt,m_primitive,m_primitiveType );
							FreezeModellingStack( m_primitive );
							if ( m_primitiveType == L"Cube")
								FreezeCubeScale( m_primitive );
						}
						in_ctxt.SetCursor(siCrossCursor);
					}	// if (m_MenuSel == kPrimMenu)
					else 
						if ( m_MenuSel == kLightPrimMenu )
						{
							KinematicState lightKine = m_lightRig.IsValid() ? m_lightRig.GetLight().GetKinematics().GetGlobal() : m_light.GetKinematics().GetGlobal();
							if ( lightKine.IsValid() )
							{
								m_lastPrimType = kLight;
								m_lastLightprimitive = m_lightRig.IsValid() ? m_lightRig.GetLight() : m_light;
								in_ctxt.EndParameterUpdate( lightKine.GetParameter(L"posx") );
								in_ctxt.EndParameterUpdate( lightKine.GetParameter(L"posy") );
								in_ctxt.EndParameterUpdate( lightKine.GetParameter(L"posz") );
								in_ctxt.EndParameterUpdate( lightKine.GetParameter(L"rotx") );
								in_ctxt.EndParameterUpdate( lightKine.GetParameter(L"roty") );
								in_ctxt.EndParameterUpdate( lightKine.GetParameter(L"rotz") );
								Light l = m_lightRig.IsValid() ? m_lightRig.GetLight() : m_light;
								if (m_arealight)	{
								
									in_ctxt.EndParameterUpdate( l.GetActivePrimitive().GetParameter(L"LightAreaXformSX") );
									in_ctxt.EndParameterUpdate( l.GetActivePrimitive().GetParameter(L"LightAreaXformSY") );
									in_ctxt.EndParameterUpdate( l.GetActivePrimitive().GetParameter(L"LightAreaXformSZ") );
								}
							
								//	***********************
								//  Restore cone visibility (camera prop)
								//	***********************
								if (  GetFromPrefs(L"Showlightscone")  )	{
									CValue result;
									CValueArray args;
									args.Add( CValue( -1 ) );
									if ( Application().ExecuteCommand( L"GetViewCamera", args, result ) == CStatus::OK ) {
										Camera l_cam = result;
										Property p;	l_cam.GetPropertyFromName(L"camvis",p);		
										p.PutParameterValue(L"attrselcones", m_conevis );
									}
								}
							}
							//	***********************
							in_ctxt.SetCursor(siCrossCursor);
						}	// if ( m_MenuSel == kLightPrimMenu )	{
				}
			//pop U image on merge button
			if ( m_additionalButtons.IsSelected( 0 ) )
				m_additionalButtons.SetSelected(0,false);
		}
		if ( in_ctxt.IsRightButtonDown() )
		{
				if(m_toolMode == kMenu)
					in_ctxt.SetCursor(siCrossCursor);	//switch to standard cursor on menu exit
		}
		m_toolMode = kIdle;
		m_activeAxis = kNoDragAxis;
		in_ctxt.Redraw();
		return CStatus::OK;
	}
	// ******************************************
	CStatus FreezeModellingStack( X3DObject& l_primitive )
	{
		CValue freeze;
		if ( Application().GetPreferences().GetPreferenceValue(L"QuickPrimitivePreferences.Freeze", freeze) != CStatus::OK )
			return CStatus::Fail;
		if ((bool)freeze != true)
			return CStatus::Abort;								
		CValueArray in_args(3);
		CValue out_val;
		in_args[0] = l_primitive.GetFullName();
		in_args[1] = L"";
		in_args[2] = L"";
		
		Application().ExecuteCommand(L"FreezeModeling",in_args,out_val);
		return CStatus::OK;
	}
	CStatus FreezeCubeScale( X3DObject& l_primitive )
	{
		CValue freeze;
		if ( Application().GetPreferences().GetPreferenceValue(L"QuickPrimitivePreferences.FreezeCubeScale", freeze) != CStatus::OK )
			return CStatus::Fail;
		if ((bool)freeze != true)
			return CStatus::Abort;								
		CValueArray in_args(1);
		CValue out_val;

		CValue cnscomp, chldcomp;
		
		in_args[0] = L"SI3D_CONSTRAINT_COMPENSATION_MODE";
		Application().ExecuteCommand(L"GetUserPref",in_args,cnscomp);
		in_args[0] = L"SI3D_NODETRANSFORM_CHILD_COMPENSATE";
		Application().ExecuteCommand(L"GetUserPref",in_args,chldcomp);
		in_args.Resize(2);
		in_args[0] = L"SI3D_CONSTRAINT_COMPENSATION_MODE";
		in_args[1] = CValue(1);
		Application().ExecuteCommand(L"SetUserPref",in_args,out_val);
		in_args[0] = L"SI3D_NODETRANSFORM_CHILD_COMPENSATE";
		Application().ExecuteCommand(L"SetUserPref",in_args,out_val);	
		in_args.Resize(4);
		in_args[0] = l_primitive.GetFullName();
		in_args[1] = siCtr;
		in_args[2] = siScl;
		in_args[3] = siXYZ;
		Application().ExecuteCommand(L"ResetTransform",in_args,out_val);

		in_args.Resize(2);
		in_args[0] = L"SI3D_CONSTRAINT_COMPENSATION_MODE";
		in_args[1] = cnscomp;
		Application().ExecuteCommand(L"SetUserPref",in_args,out_val);
		in_args[0] = L"SI3D_NODETRANSFORM_CHILD_COMPENSATE";
		in_args[1] = chldcomp;
		Application().ExecuteCommand(L"SetUserPref",in_args,out_val);

		return CStatus::OK;
	}
	CStatus AddProjectionToPrimitive(ToolContext& in_ctxt, X3DObject& l_primitive, CString l_primitiveType)
	{
		//int cuvs;
		if ( l_primitiveType == L"Circle" || l_primitiveType == L"Null" )
			return CStatus::False;
		CValue cuvs;
		if ( Application().GetPreferences().GetPreferenceValue(L"QuickPrimitivePreferences.CreateUVs", cuvs) != CStatus::OK )
			return CStatus::Fail;	
		
		if ((bool)cuvs != true)
			return CStatus::Abort;								
		CValueArray in_args(8);
		CValue out_val;
		in_args[0] = l_primitive.GetFullName();
		if (m_primitiveType == L"Grid" || m_primitiveType == L"Disc")	{
			in_args[1] = siTxtPlanarXZ;
			in_args[2] = siTxtDefaultPlanarXZ;
		}
		else
			if ( l_primitiveType == L"Sphere" || l_primitiveType == L"Dodecahedron") 	{
					in_args[1] = siTxtSpherical;
					in_args[2] = siTxtDefaultSpherical;
				}
				else
					if ( l_primitiveType == L"Cone" || l_primitiveType == L"Cylinder" || l_primitiveType == L"Torus") 	{
						in_args[1] = siTxtCylindrical;
						in_args[2] = siTxtDefaultCylindrical;
					}
					else
						if ( l_primitiveType == L"Cube" )	{
						in_args[1] = siTxtCubic;
						in_args[2] = siTxtDefaultCubic;
					}
		in_args[3]=in_args[4]=in_args[5]=in_args[6]=in_args[7]=L"";
		Application().ExecuteCommand(L"CreateProjection",in_args,out_val);

		return CStatus::OK;
	}
	//********************* Merge primitive to polymesh, using MeshMerge command
	CStatus MergePrimitiveToSelected(ToolContext& in_ctxt, CRefArray &l_prim, X3DObject &l_out_obj, bool deleteInputs = true)
	{
		CString l_tobjNames=L"";
		
		for (int i=0;i<l_prim.GetCount();i++ )	{
			X3DObject l_tobj = l_prim[i];
			if ( l_tobj.GetName() != L"" && l_tobj.IsValid() && l_tobj.GetType() == L"polymsh" )
				l_tobjNames += L","  + l_tobj.GetFullName();
		}
		if (l_tobjNames == L"")
			return CStatus::False;

		Selection sel( Application().GetSelection() );
		CRefArray l_selobjs = sel.GetArray();
		CRefArray l_objs = ExpandSelection(in_ctxt, l_selobjs);							//expand components to objects
		if ( l_objs.GetCount() != 1)
			return CStatus::False;

		X3DObject l_obj( l_objs[0] );
		if ( !l_obj.IsValid() || l_obj.GetType() != L"polymsh" )
			return CStatus::False;
		// get children of input selection and reparent it scene root
		CRefArray l_objChildren = l_obj.GetChildren();
		Model oRoot = Application().GetActiveSceneRoot();
		oRoot.AddChild(l_objChildren);
		// apply mesh merge op
		CString l_meshName = l_obj.GetName();	//save mesh name
		CValue retval;
		CValueArray args(6);
		args[0] = L"MeshMerge";
		args[1] = L"";
		args[2] = l_obj.GetFullName() + l_tobjNames;			
		args[3] = CValue(3);
		args[4] = L"siPersistentOperation";
		args[5] = L"siKeepGenOpInputs";
		Application().ExecuteCommand( L"ApplyGenOp", args, retval );
		CValueArray ops(retval);
		Operator genOp = ops[0];
		genOp.PutParameterValue(L"tolerance", 0.0);
		//get op x3d object
		X3DObject l_parent = l_obj.GetParent3DObject();
#ifdef _DEBUG
		logmsg(L"genop",genOp.GetFullName() );
		logmsg(L"parent", l_parent.GetFullName() );
#endif
		//transfer uvs etc
		args.Resize(4);
		args[0] = genOp.GetFullName();
		args[1] = genOp.GetParent3DObject().GetFullName();
		args[2] = CValue(L"");
		args[3] = CValue(true);

		Application().ExecuteCommand( L"TransferAllPropertiesAcrossGenOp", args, retval );
		
		l_out_obj = genOp.GetParent3DObject();
		
		//cleanup, delete input objects
		if (deleteInputs)	{
			args.Resize(1);
			args[0] = genOp.GetFullName();
			Application().ExecuteCommand( L"DeleteGeneratorInputs",args,retval);
		}
		// change name to original selected mesh
		l_out_obj.PutName(l_meshName);	
		//reparent output object to original parent of selected mesh
		CRefArray childs;
		childs.Add( l_out_obj.GetRef() );
		l_parent.AddChild( childs );
		// parent back children of original selected object
		l_out_obj.AddChild( l_objChildren );
		logmsg(L"Primitive merged to selection");
		return CStatus::OK;
	}
	// ************************************************************************************************************************************************ DrawPrimitiveInfo
	CStatus DrawPrimitiveInfo(ToolContext& in_ctxt)
	{
			LONG viewWidth, viewHeight;

			if ( in_ctxt.GetViewSize( viewWidth, viewHeight ) != CStatus::OK )
				return CStatus::Fail;
			in_ctxt.BeginViewDraw();
			glColor3f( 1.0,1.0, 1.0 );
			LONG width = 0, height = 0, descent = 0;
			CString l_szprim=L"";
				
			if (m_dragMode == kFirst){
				if(m_paramCnt == 1)
					l_szprim = m_parameter[0].GetName() + L": "+ CValue( floor((double)m_parameter[0].GetValue()*1000.0)/1000.0 ).GetAsText();
				else{
					if (m_primitiveType != L"Cube")	{
						l_szprim = m_parameter[0].GetName() + L": "+ CValue( floor((double)m_parameter[0].GetValue() *1000.0)/1000.0 ).GetAsText();;
						l_szprim += L" "+m_parameter[1].GetName() + L": "+ CValue( floor((double)m_parameter[1].GetValue() *1000.0)/1000.0 ).GetAsText();
					} else	{
						l_szprim = L"U Length: "+ CValue( floor( ( (double)m_parameter[0].GetValue() / 100.0 ) *1000.0)/1000.0 ).GetAsText();;
						l_szprim += L" V Length"+m_parameter[1].GetName() + L": "+ CValue( floor( ( (double)m_parameter[1].GetValue() / 100.0 ) *1000.0)/1000.0 ).GetAsText();
					}
				}
			}
			else{
				if(m_paramSecCnt == 1)	{
					if (m_primitiveType != L"Cube")	
						l_szprim = m_parameterSec[0].GetName() + L": "+ CValue( floor((double)m_parameterSec[0].GetValue()*1000.0)/1000.0 ).GetAsText();
					else
						l_szprim = L"Height: "+ CValue( floor( ( ( double )m_parameterSec[0].GetValue() / 100.0 )*1000.0)/1000.0 ).GetAsText();
				}
				else{
					l_szprim = m_parameterSec[0].GetName() + L": "+ CValue( floor((double)m_parameterSec[0].GetValue()*1000.0)/1000.0 ).GetAsText();
					l_szprim += L" "+m_parameterSec[1].GetName() + L": "+ CValue( floor((double)m_parameterSec[1].GetValue()*1000.0)/1000.0 ).GetAsText();
				}
			}
			in_ctxt.GetTextSize( l_szprim, width, height, descent );
			GLint x = (GLint)m_mouseDragX - width/2, y = (GLint)m_mouseDragY - 27/*pixels*/;
			glRasterPos2i(x ,y);
			
			in_ctxt.DrawTextString(l_szprim);

			in_ctxt.EndViewDraw();
			return CStatus::OK;
	}
	// ************************************************************************************************************************************************ DRAW Tool Info
	CStatus DrawToolInfo(ToolContext& in_ctxt)
	{
			LONG viewWidth, viewHeight;

			if ( in_ctxt.GetViewSize( viewWidth, viewHeight ) != CStatus::OK )
				return CStatus::Fail;
			CString szTool = L"";
			if (m_MenuSel == kPrimMenu)
				szTool = m_piemenu.GetSelectionName();
			else if (m_MenuSel == kLightPrimMenu )
				szTool = m_piemenuLights.GetSelectionName();
			in_ctxt.BeginViewDraw();
			glColor3f( 1.0,1.0, 1.0 );
			LONG width = 0, height = 0, descent = 0;
			in_ctxt.GetTextSize( szTool, width, height, descent );
			GLint x = (GLint)m_mouseX - width/2, y = (GLint)m_mouseY - 27/*pixels*/;
			glRasterPos2i(x ,y);
			
			in_ctxt.DrawTextString( szTool );

			in_ctxt.EndViewDraw();
			return CStatus::OK;
	}
	CStatus DrawToolHelp(ToolContext& in_ctxt)
	{
			LONG viewWidth, viewHeight;

			if ( in_ctxt.GetViewSize( viewWidth, viewHeight ) != CStatus::OK )
				return CStatus::Fail;
			CString szTool = L"";
			if (m_MenuSel == kLightPrimMenu)
				szTool =L"Hold SHIFT to adjust light's height. Hold RMB to adjust Area light's area";
			else if (m_MenuSel == kPrimMenu )
				szTool = L"Hold SHIFT to adjust primitive's height";
			in_ctxt.BeginViewDraw();
			glColor3f( 1.0,1.0, 1.0 );
			LONG width = 0, height = 0, descent = 0;
			in_ctxt.GetTextSize( szTool, width, height, descent );
			
			GLint x = viewWidth/2 - width/2, y = 60/*pixels*/;
			glRasterPos2i(x ,y);
			
			in_ctxt.DrawTextString( szTool );

			in_ctxt.EndViewDraw();
			return CStatus::OK;
	}
	// ************************************************************************************************************************************************ DRAW Highlight

	CStatus DrawComponentHighlight(ToolContext& in_ctxt)
	{
		Property oGApprox;
		if(!m_picked.IsValid())
			return CStatus::Fail;
		//get subdiv level and subdivision method
		m_picked.GetPropertyFromName("geomapprox", oGApprox);
		int approxLevel = oGApprox.GetParameter("gapproxmosl").GetValue();
		int subdrule = m_picked.GetActivePrimitive().GetParameterValue("subdrule");
		
		CMatrix4 l_pickedMatrix(m_picked.GetKinematics().GetGlobal().GetTransform().GetMatrix4() );
		
		glColor4ub( highlightColorR, highlightColorG, highlightColorB,highlightColorA );			
		
		if (m_pickedSubcompType==kEdge) //draw edge
		{
			CVector3Array ve;
			if (approxLevel && (subdrule != 3) ) // approxlevel not 0 and subd rule not linear
				ve = subdivideEdge(m_pickedEdge, (int)1);
			else 
				ve =  m_pickedEdge.GetVertices().GetPositionArray ();

			glLineWidth(2.0f);
			glEnable(GL_BLEND);
			glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);

			glEnable(GL_LINE_SMOOTH);
			glBegin( GL_LINE_STRIP );
				for (LONG i=0;i< ve.GetCount();i++)
				{
					ve[i]*=l_pickedMatrix;
					glVertex3d( ve[i].GetX(),ve[i].GetY(),ve[i].GetZ() );
				}
			glEnd();
		
			glDisable(GL_LINE_SMOOTH);	
			glDisable(GL_BLEND);
		}
		else 
		if (m_pickedSubcompType==kPoly) //poly
		{								
			glEnable( GL_BLEND );
			glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
			glEnable ( GL_POLYGON_STIPPLE );
			glPolygonStipple(halftonePolyMask);
			
			if (approxLevel && (subdrule != 3) )		//subdivided poly drawing
			{
				CVector3Array ppos = subdivideFace(m_pickedPolyFace,1);
				CVector3 pos;
				glBegin(GL_TRIANGLE_FAN);
					for(int i=0;i<ppos.GetCount();i++)
					{
						pos = ppos[i];
						pos*=l_pickedMatrix;
						glVertex3d( pos.GetX(),pos.GetY(),pos.GetZ() );
					}
					pos = ppos[1];
					pos*=l_pickedMatrix;
					glVertex3d( pos.GetX(),pos.GetY(),pos.GetZ() );
				glEnd();
			}
			else
				{
					LONG nbTriangles = m_pickedPolyFace.GetNbPoints()-2;
					CLongArray polygonNeiVtxIndices = m_pickedPolyFace.GetVertices().GetIndexArray();
					CLongArray polygonTriSubIndices = m_pickedPolyFace.GetTriangleSubIndexArray();
					PolygonMesh pmesh( m_picked.GetActivePrimitive().GetGeometry() );				
					CVertexRefArray  verts( pmesh.GetVertices() );
									
					CVector3 pos;
					glBegin(GL_TRIANGLES);
						LONG j;
						for(j = 0; j < nbTriangles; j++) 
							for(int k=0;k<3;k++) 
							{
								LONG index = polygonNeiVtxIndices[polygonTriSubIndices[j*3+k]];
								Vertex v(verts[index]);
								CVector3 pos(v.GetPosition()); 
													
								pos*=l_pickedMatrix;	
								glVertex3d( pos.GetX(),pos.GetY(),pos.GetZ() );
							}
					glEnd();
				}
							
			glDisable ( GL_POLYGON_STIPPLE );
			glDisable ( GL_BLEND );					
		}
		else	
			if (m_pickedSubcompType==kVertex) 
			{
				CVector3 pos;
				if (approxLevel && (subdrule != 3) )
					pos = subdividePoint(m_pickedVertex,(int) 1);
				else
					pos = m_pickedVertex.GetPosition();
				pos*=l_pickedMatrix;
				glPointSize(8.0f);
				glEnable(GL_POINT_SMOOTH);
				glBegin( GL_POINTS );
						glVertex3d( pos.GetX(),pos.GetY(),pos.GetZ() );
				glEnd();
				glDisable(GL_POINT_SMOOTH);
			}
		return CStatus::OK;
	}
	
	CStatus DrawAxisPlane(ToolContext& in_ctxt,CVector3 center, double l_scale, CLine x, CLine y, CLine z)
	{
		const double pos_start = 2;
		CVector3 dir = z.GetTangent();
		dir.NormalizeInPlace();
		CVector3 pos = x.GetPosition(-1);
		CVector3 pos2 = x.GetPosition(1);
		CLine pt1(x.GetPosition(-1), dir );
		CLine pt2(x.GetPosition(1), dir );

		CVector3 points[4];
		
		points[0] = pt2.GetPosition(1);
		points[1] = pt2.GetPosition(-1);
		points[2] = pt1.GetPosition(-1);
		points[3] = pt1.GetPosition(1);
		
		glBegin(GL_LINE_LOOP);
			for(int i=0;i<4;i++) {
				
				points[i]-=center;
				points[i].ScaleInPlace(l_scale);
				points[i]+=center;
				glVertex3d(points[i].GetX(),points[i].GetY(),points[i].GetZ() );
			}
		glEnd();
		glEnable ( GL_POLYGON_STIPPLE );
		glBegin(GL_QUADS);
			for(int i=0;i<4;i++)
				glVertex3d(points[i].GetX(),points[i].GetY(),points[i].GetZ() );
		glEnd();
		glDisable ( GL_POLYGON_STIPPLE );
		return CStatus::OK;
	}

	CStatus DrawPrimitivesHandles(ToolContext& in_ctxt)
	{
		if(!m_primitive.IsValid() )
			return CStatus::Fail;
			
		X3DObject l_obj(m_primitive);
			
		CTransformation trans = l_obj.GetKinematics().GetGlobal().GetTransform();
		CTransformation trans2 = trans;
		double minx,miny,mz,maxx,maxy,maxz;

		CTransformation l_t;
		l_obj.GetBoundingBox(minx,miny,mz,maxx,maxy,maxz,l_t);
		maxy+=(maxy*0.1);
		miny+=(miny*0.01);
				
		trans.AddLocalTranslation(CVector3(0,maxy,0));
		trans2.AddLocalTranslation(CVector3(0,miny,0));
			
		CMatrix4 l_omatrix = trans.GetMatrix4();
		GLdouble mat[16],matMinY[16];
				
		trans2.SetScalingFromValues(1.0, 1.0, 1.0); 
		trans.SetScalingFromValues(1.0, 1.0, 1.0);
		trans.GetMatrix4().Get(mat[0],  mat[1],  mat[2],  mat[3],	mat[4],  mat[5],  mat[6],  mat[7],	mat[8],  mat[9],  mat[10], mat[11],	mat[12], mat[13], mat[14], mat[15]	);
		trans2.GetMatrix4().Get(matMinY[0],  matMinY[1],  matMinY[2],  matMinY[3],	matMinY[4],  matMinY[5],  matMinY[6],  matMinY[7],	matMinY[8],  matMinY[9],  matMinY[10], matMinY[11],	matMinY[12], matMinY[13], matMinY[14], matMinY[15]	);
			
		double l_dScale = GetViewScaleFactor( in_ctxt,trans.GetTranslation(), 1 );	//scale factor for elements that needs to have constant scale in view
	
		//box at end of cube
				
		/*
		GLfloat blackColor[] = { 0.0, 0.0, 0.0, 1.0 };
		GLfloat diffuseColor[] = { 0,1.0, 0, .8};
		GLfloat ambientColor[] = { diffuseColor[0]*0.2f, diffuseColor[1]*0.2f, diffuseColor[2]*0.2f, diffuseColor[3] };
		glMaterialfv( GL_FRONT, GL_AMBIENT, ambientColor );
		glMaterialfv( GL_FRONT, GL_DIFFUSE, diffuseColor );
		glMaterialfv( GL_FRONT, GL_SPECULAR, blackColor );
		glMaterialf(  GL_FRONT, GL_SHININESS, 0.0 );
		glEnable( GL_LIGHTING );
		InitHeadlight(in_ctxt);
				
		glPushMatrix();
		glMultMatrixd(mat);	
		float size = 5 * static_cast<float>(l_dScale);
		if (size > 0.0001)
			ogl_drawCube(size);
				
		glPopMatrix();
			
		glDisable( GL_LIGHTING );*/

		glPushMatrix();
		glMultMatrixd(matMinY);	
				
		glLineWidth(3.0f);
		double circleSize = (maxx-minx)*0.5;
		if(m_piemenu.GetSelectionName() == L"Cube")
			circleSize *= sqrt(2.0);
		circleSize += .1;
				
		ogl_drawCircle(circleSize,50, true);
		//GLubyte cc[4]={200,200,0,120};
		//GLubyte co[4]={200,200,0,110};
				
		//ogl_drawDisc(cc,co,circleSize,50,true);
		glPopMatrix();
				
			
	//	TermHeadlight(in_ctxt);
			
			
		return CStatus::OK;
	}

	void	DrawLightHelperTriangle()	
	{
		// ***************************
		//		draw light helpers
		// ***************************
		CVector3 l_basePos = m_gridPlane.Project( m_lightPos );
		glEnable( GL_BLEND );
		glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
		glEnable ( GL_POLYGON_STIPPLE );
		glPolygonStipple(halftonePolyMask);
					
		glColor4d( 1.0, 0.5, 0.0, 0.4 );

		glBegin( GL_TRIANGLES );
			glVertex3d( m_intPos.GetX(), m_intPos.GetY(), m_intPos.GetZ() );
			glVertex3d( l_basePos.GetX(), l_basePos.GetY(), l_basePos.GetZ() );
			glVertex3d( m_lightPos.GetX(), m_lightPos.GetY(), m_lightPos.GetZ() );
		glEnd();
		glDisable ( GL_POLYGON_STIPPLE );
		glEnable( GL_LINE_SMOOTH );
		glLineStipple(3, 0xAAAA );
		glEnable( GL_LINE_STIPPLE );
		glColor4d( 1.0, 0.6, 0.0, 0.7 );

		glBegin( GL_LINE_LOOP );
			glVertex3d( m_intPos.GetX(), m_intPos.GetY(), m_intPos.GetZ() );
			glVertex3d( l_basePos.GetX(), l_basePos.GetY(), l_basePos.GetZ() );
			glVertex3d( m_lightPos.GetX(), m_lightPos.GetY(), m_lightPos.GetZ() );
		glEnd();
		glDisable( GL_LINE_STIPPLE );
		glDisable ( GL_LINE_SMOOTH );
		glDisable ( GL_BLEND );	

		glBegin( GL_LINE_LOOP );
		for(int i=0;i< 4;i++)
			glVertex3d( m_dumpvector[i][0], m_dumpvector[i][1],m_dumpvector[i][2]);
			
			
		glEnd();
	}
	// ************************************************************************************************************************************************ DRAW
	CStatus Draw( ToolContext& in_ctxt )
	{	
		if ( in_ctxt.IsActiveView())
		{
			if (m_toolMode == kMenu)
			{
				//draw menu line
				in_ctxt.BeginViewDraw();
					glColor4ub(255,200,0,200);
					ogl_drawLine(m_mouseDownX,m_mouseDownY,0,m_mouseDragX,m_mouseDragY,0,true);
				in_ctxt.EndViewDraw();
				//draw menu
				m_piemenu.Draw(in_ctxt);
				m_piemenuLights.Draw(in_ctxt);
			}
			else
				if(m_toolMode == kPrimDrawing)
				{	
					if ( m_MenuSel != kLightPrimMenu )
						DrawPrimitiveInfo(in_ctxt);					
					else	
						DrawLightHelperTriangle();
					DrawToolHelp( in_ctxt );
				}		// 	if(m_toolMode == kPrimDrawing)
			LONG l_index;
			if ( /*!in_ctxt.IsInteracting() &&*/ !in_ctxt.IsNavigating() && m_toolMode != kMenu && 
				(
				(!m_alignButtons.IsPointInside(in_ctxt,m_mouseX,m_mouseY, l_index) )&&
				(!m_additionalButtons.IsPointInside(in_ctxt,m_mouseX,m_mouseY, l_index) )&&
				(!m_axisButtons.IsPointInside(in_ctxt,m_mouseX,m_mouseY,l_index) )&&
				(!m_toolscopeButtons.IsPointInside(in_ctxt,m_mouseX,m_mouseY,l_index) ) 
				) && m_pickIsValid
				)
			{
				CVector3 l_pos=m_pickedSubcompPos;
				CVector3 x,y,z;
				CLine axis[3];
				CVector3 normal = m_pickedNormal;
				bool gridpicking = false;
				
				if (m_pickedSubcompType != kNone && !in_ctxt.IsInteracting() )
						DrawComponentHighlight(in_ctxt);
				
				if (m_alignButtons.IsSelected(m_pickedSubcompType) )
				{			
					AlignPrimitiveToComponentRefFrame(in_ctxt,m_picked,x,y,z,false);
					
					axis[0].Set(l_pos,x);
					axis[1].Set(l_pos,y);
					axis[2].Set(l_pos,z);
				}
				
				else 
					if( m_picked.GetType() == L"null")
					{
						CTransformation transf= m_picked.GetKinematics().GetGlobal().GetTransform();	
						CVector3 t= transf.GetTranslation();
						CMatrix3 m = transf.GetRotationMatrix3();
						axis[0].Set(t,CVector3(1,0,0));
						axis[1].Set(t,CVector3(0,1,0));
						axis[2].Set(t,CVector3(0,0,1));
						axis[0].MulByMatrix3InPlace(m);	axis[1].MulByMatrix3InPlace(m);	axis[2].MulByMatrix3InPlace(m);
						axis[0].SetOrigin(t);axis[1].SetOrigin(t);axis[2].SetOrigin(t);
						l_pos = t;
						
					}
					else if(m_picked.GetType() == "crvlist" || m_picked.GetType() == L"surfmsh") {
						axis[0].Set(l_pos,m_pickedCurveFrameAxisX);
						axis[1].Set(l_pos,m_pickedCurveFrameAxisY);
						axis[2].Set(l_pos,m_pickedCurveFrameAxisZ);
					}
					else //grid
						{
							gridpicking = true;

							CVector3 l_cross;

							axis[0].Set(l_pos, m_pickedFrameAxisX);
							axis[1].Set(l_pos, m_pickedFrameAxisY);
							axis[2].Set(l_pos, m_pickedFrameAxisZ);
									
						}

				//draw axes			
				if( m_picked.IsValid() || gridpicking)
				{
					double l_dScale = GetViewScaleFactor( in_ctxt,l_pos, 1 );	//scale factor for elements that needs to have constant scale in view
					
					glEnable (GL_BLEND);
					glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
					glEnable ( GL_CULL_FACE );
					glDisable ( GL_DEPTH_TEST );
					glDepthMask( GL_FALSE );
					if (in_ctxt.IsInteracting() )	{
						if (m_MenuSel == kPrimMenu)	{
							glColor4ub(200,200,0,50);
							if (m_piemenu.GetSelectionName() != L"Null" && m_piemenu.GetSelectionName() != L"Grid"  && m_piemenu.GetSelectionName() != L"Cube")
								DrawPrimitivesHandles(in_ctxt);
						}
					}
					else	{
						glColor4ub(200,200,0,150);
						glLineWidth(3.5f);
						
						switch (m_axisButtons.GetSelectionIndex() )	{
							case 0:DrawAxisPlane(in_ctxt,l_pos,l_dScale*m_planeScale,axis[2],axis[0],axis[1]);break;
							case 2:DrawAxisPlane(in_ctxt,l_pos,l_dScale*m_planeScale,axis[1],axis[2],axis[0]);break;
							case 1:DrawAxisPlane(in_ctxt,l_pos,l_dScale*m_planeScale,axis[0],axis[1],axis[2]);break;
						};
					}
					glLineWidth(1.5f);
					
					double axis_len = m_axisScale * l_dScale; //axis drawing length / pix/ 
					const double cone_len = m_axisScale*0.3 * l_dScale; //cone scale
					
					GLdouble mat[16];
					for(int i=0;i<3;i++)
					{
						int colors[4]={0,0,0,0};
						if ( m_axisButtons.GetSelectionIndex() == i ) 	
							colors[3] = 255;					
						else 						
							colors[3] = 50;
						
						CVector3 v  = axis[i].GetPosition(axis_len);
						CVector3 up = axis[i].GetTangent();
						//-----------------------draw lines
						
						colors[i] = 255;
						
						glColor4ub(colors[0],colors[1],colors[2],colors[3]);
						glBegin(GL_LINES);
							glVertex3d(l_pos[0],l_pos[1],l_pos[2]);
							glVertex3d(v[0],v[1],v[2]);
						glEnd();	

						glPushMatrix();
						//-----------------------draw end cones
						CTransformation tr;
						CVector3 r_obj= m_picked.GetLocalScaling();
						CRotation rot;
						
						switch (i){
							case 0: rot.SetFromXYZAxes( axis[2].GetTangent(),axis[0].GetTangent(),axis[1].GetTangent() );break;
							case 1:	rot.SetFromXYZAxes( axis[0].GetTangent(),axis[1].GetTangent(),axis[2].GetTangent() );break;
							case 2:	rot.SetFromXYZAxes( axis[1].GetTangent(),axis[2].GetTangent(),axis[0].GetTangent() );break;
						}
						tr.SetTranslation(v);					
						tr.SetRotation(rot);							
						tr.SetScalingFromValues(cone_len, cone_len*2, cone_len);
						tr.GetMatrix4().Get(mat[0],  mat[1],  mat[2],  mat[3],	mat[4],  mat[5],  mat[6],  mat[7],	mat[8],  mat[9],  mat[10], mat[11],	mat[12], mat[13], mat[14], mat[15]	);
						glMultMatrixd(mat);
							
						ogl_drawCone();
							
						glPopMatrix();
					}
					glDisable ( GL_CULL_FACE );
					glEnable ( GL_DEPTH_TEST );
					glDepthMask( GL_TRUE );
					
					glDisable( GL_BLEND);
					
				}

				//info
				if(m_toolMode != kPrimDrawing)
					DrawToolInfo(in_ctxt);		
			}
			// draw buttons
			if ( !in_ctxt.IsInteracting() || m_toolMode == kButtonDrag )
			{			
				m_alignButtons.Draw(in_ctxt);
				m_axisButtons.Draw(in_ctxt);
				m_toolscopeButtons.Draw(in_ctxt);
				m_additionalButtons.Draw(in_ctxt);
			}
		}
		
		LogOpenGLErrors();
		return CStatus::OK;
	}

	//**************************************************************************************************************** Mouse MOVE
	CStatus MouseMove( ToolContext& in_ctxt )
	{
		// Called when mouse is moved without any buttons pressed
		CStatus l_status;
		in_ctxt.GetMousePosition( m_mouseX, m_mouseY);
		
		clock_t now=clock() - m_tooltime;
		
		if (now>MOUSE_MOVE_TIME_DELAY) {
			l_status = GetSnapData(in_ctxt,m_mouseX,m_mouseY,m_pickedSubcompPos);
			m_pickIsValid = l_status == CStatus::OK ? true : false;
			
			if (l_status == CStatus::OK || m_lastMouseMoveStatus == CStatus::OK)
				in_ctxt.Redraw(false);
			m_tooltime=clock();
			m_lastMouseMoveStatus = l_status;
		};
		
		return CStatus::OK;
	}
	CStatus MouseEnter( ToolContext &in_ctxt )
	{
		//in_ctxt.Redraw(true);
		return CStatus::OK;
	}
	CStatus MouseLeave( ToolContext &in_ctxt )
	{
		//in_ctxt.Redraw(true);
		return CStatus::OK;
	}
	CStatus SnapValid( ToolContext& in_ctxt )
	{
		if (m_primitive.IsValid())
		{
			X3DObject xobj = in_ctxt.GetSnapObject();
			if(xobj.GetFullName() == m_primitive.GetFullName() )
				return CStatus::Abort;
		}
		return CStatus::OK;
	}
	bool IsModKeyUp(ToolContext &in_ctxt)
	{
		return in_ctxt.IsRightButtonDown();
	}

	void constructCircle (LONG in_segments, double in_r, CLongArray &out_points) {
		out_points.Clear();
		int i = 0;
		const double kRadianIncr = 6.28f/in_segments;
		for (i = 0; i < in_segments; i++)
		{
			double angleRadians = kRadianIncr*i;
			out_points.Add(LONG(in_r * cos(angleRadians) ));
			out_points.Add(LONG(in_r * sin(angleRadians)));
		}
	}

	/*
	function ApplyShader(primitive, shaderType, shaderName, connectionPoint)
{
   var arnoldPlugin = Application.plugins("Arnold Lights");
   var preset = arnoldPlugin.OriginPath + "/../../Data/DSPresets/Shaders/" + shaderType + "/" + shaderName + ".Preset";
   var shader = CreateShaderFromPreset(preset, primitive, null);
   SIConnectShaderToCnxPoint(shader, primitive + "." + connectionPoint, true);
}
	*/
	Shader ApplyShader(CString plugin_name, Primitive prim, CString shaderType, CString shaderName, CString connection_point)
	{
		Plugin plugin;
		plugin = Application().GetPlugins().GetItem( plugin_name );
		if ( !plugin.IsValid())	{
			logmsg(L"plugin not found");
			return Shader();
		}		
		CString  preset = plugin.GetOriginPath() + L"/../../Data/DSPresets/Shaders/" + shaderType + L"/" + shaderName + L".Preset";
		CValue outArg;
		CValueArray inArgs;
		inArgs.Add( preset );
		inArgs.Add( prim.GetFullName() );
		inArgs.Add( L"" );
		Application().ExecuteCommand(L"CreateShaderFromPreset", inArgs, outArg);
		Shader out_shader = outArg;
		inArgs.Clear();
		inArgs.Add( outArg );
		inArgs.Add( CString( prim.GetFullName() + L"." + connection_point ) );
		inArgs.Add( true );
		Application().ExecuteCommand(L"SIConnectShaderToCnxPoint", inArgs, outArg);
		return out_shader;
	}
	//********************************************************************************************************** Update Primitive
	CStatus UpdateLightPrimitive(ToolContext &in_ctxt)
	{
			// Called when mouse is moved with a mouse button pressed
		LONG x, y;
		in_ctxt.GetMousePosition( x, y );

		if ( in_ctxt.IsLeftButtonDown() && !in_ctxt.IsRightButtonDown() )
		{
			if (!in_ctxt.IsShiftKeyDown() || m_vertAxis.GetLengthSquared() < MicroEPS)
			{
				// Move in plane
				in_ctxt.SetCursor( siCrossCursor );

				CVector3 l_Pos;
				in_ctxt.EnableSnapping( true );
				if ( in_ctxt.GetWorldPosition( x, y, l_Pos ) != CStatus::OK )
					return CStatus::OK;

				// Reject points behind the camera
				CVector3 l_cameraPos;
				if ( in_ctxt.WorldToCamera( l_Pos, l_cameraPos ) != CStatus::OK || l_cameraPos.GetZ() > 0.0 )
					return CStatus::OK;

				// Update position
				m_lightPos = l_Pos;
			}
			else
			{
				// Calculate vertical offset
				in_ctxt.SetCursor( siDragYCursor );

				CLine l_Ray;
				in_ctxt.EnableSnapping( false );
				if ( in_ctxt.GetWorldRay( x, y, l_Ray ) != CStatus::OK )
					return CStatus::OK;

				// Calculate vertical offset
				CLine l_Vert( m_lightPos, m_vertAxis );
				CVector3 l_Pos = l_Vert.GetClosestPoint( l_Ray );
				CVector3 l_Off = l_Pos; 
				l_Off -= m_lightPos;
				// Update position
				m_lightPos = l_Pos;
				// Update manipulation plane
				CVector3 l_NewOrigin = m_manipPlane.GetOrigin();
				m_manipPlane.SetOrigin( l_NewOrigin += l_Off );
				in_ctxt.SetManipulationPlane( m_manipPlane );
			}
			KinematicState lightKine= m_lightRig.IsValid() ? m_lightRig.GetLight().GetKinematics().GetGlobal() : m_light.GetKinematics().GetGlobal();
			in_ctxt.UpdateParameter( lightKine.GetParameter(L"posx"), m_lightPos.GetX() );
			in_ctxt.UpdateParameter( lightKine.GetParameter(L"posy"), m_lightPos.GetY() );
			in_ctxt.UpdateParameter( lightKine.GetParameter(L"posz"), m_lightPos.GetZ() );

			// Update rotation for non-rigged directional lights (lookat)
			if ( m_dirlight )	{
				CVector3 dir=m_light.GetKinematics().GetGlobal().GetTransform().GetTranslation();					
				dir-=m_lightTarget;
				dir.NormalizeInPlace();	
				CVector3 up_vect;
				CPlane vp;
				in_ctxt.GetGridPlane(vp);
				up_vect = vp.GetNormal();
				CVector3 leftv,fwv;	
				fwv = fwv.Cross( up_vect, dir );
				fwv.NormalizeInPlace();
				leftv = leftv.Cross( dir, fwv );			
				CRotation rot;
				rot.SetFromXYZAxes( fwv,leftv, dir );
				double x, y, z;
				rot.GetXYZAngles(x,y,z);
				in_ctxt.UpdateParameter( lightKine.GetParameter(L"rotx"), RadiansToDegrees( x ) );
				in_ctxt.UpdateParameter( lightKine.GetParameter(L"roty"),RadiansToDegrees( y ));
				in_ctxt.UpdateParameter( lightKine.GetParameter(L"rotz"), RadiansToDegrees( z ));
			}
		}
		else 
			if ( in_ctxt.IsRightButtonDown() )
			{
				if (m_arealight)
				{
					// *************************
					// Calc area light shape
					// *************************
					CVector3 upp;
					Light light = m_lightRig.IsValid() ? m_lightRig.GetLight() : m_light;
					upp = m_dirlight ? m_lightTarget : light.GetInterest().GetKinematics().GetGlobal().GetTransform().GetTranslation();
					upp-=light.GetKinematics().GetGlobal().GetTransform().GetTranslation();
					upp.NormalizeInPlace();	
					CPlane cp( m_lightPos, upp );
					CVector3 leftv,upv,fwv;
					anglesToAxes( light.GetKinematics().GetGlobal().GetTransform().GetRotationXYZAngles(), leftv, upv, fwv);			
					fwv = fwv.Cross( upp, m_vertAxis );
					leftv = leftv.Cross( fwv, upp );
					cp.SetTangents(fwv,leftv );
					CVector3 hitpos;
					CLine mouseray;
					in_ctxt.EnableSnapping( false );
					in_ctxt.GetWorldRay( x, y, mouseray );	
					CVector3 raynormal = mouseray.GetTangent();
					raynormal.NormalizeInPlace();
					double anglePlane = cp.GetNormal().Dot( raynormal );

					if (abs(anglePlane)< 0.015 )	
						in_ctxt.GetViewPlane(cp);
					cp.Intersect( mouseray, hitpos );

					CLine a(m_lightPos,fwv);
					CLine b(m_lightPos,leftv);				
					CLine c(hitpos,upp);

					double offseta = a.GetParameter(a.GetClosestPoint(c));
					double offsetb = b.GetParameter(b.GetClosestPoint(c));
					CVector3 hita = a.GetPosition(offseta);	hita-=m_lightPos;
					CVector3 hitb = b.GetPosition(offsetb); hitb-=m_lightPos;
				
					double pa=abs(hita.GetLength())*2;
					double pb=abs(hitb.GetLength())*2;
					Light l;
					l = m_lightRig.IsValid() ? m_lightRig.GetLight() : m_light;
					in_ctxt.UpdateParameter( l.GetActivePrimitive().GetParameter(L"LightAreaXformSX"), pa );
					in_ctxt.UpdateParameter( l.GetActivePrimitive().GetParameter(L"LightAreaXformSY"), pb );
					in_ctxt.EnableSnapping( true );
				}
			}
		return CStatus::OK;
	}
	void anglesToAxes(const CVector3 angles, CVector3& left, CVector3& up, CVector3& forward)
{
    const double DEG2RAD = 3.141593f / 180;
    double sx, sy, sz, cx, cy, cz, theta;

    // rotation angle about X-axis (pitch)
    theta = angles.GetX() * DEG2RAD;
    sx = sin(theta);
    cx = cos(theta);

    // rotation angle about Y-axis (yaw)
    theta = angles.GetY() * DEG2RAD;
    sy = sin(theta);
    cy = cos(theta);

    // rotation angle about Z-axis (roll)
    theta = angles.GetZ() * DEG2RAD;
    sz = sin(theta);
    cz = cos(theta);

    // determine left axis
    left.PutX( cy*cz );
    left.PutY( sx*sy*cz + cx*sz );
    left.PutZ( -cx*sy*cz + sx*sz );

    // determine up axis
    up.PutX(  -cy*sz );
    up.PutY( -sx*sy*sz + cx*cz );
    up.PutZ(  cx*sy*sz + sx*cz );

    // determine forward axis
    forward.PutX( sy );
    forward.PutY( -sx*cy );
    forward.PutZ( cx*cy );
}
	CStatus UpdatePrimitive(ToolContext &in_ctxt)
	{
		LONG x, y;
		CVector3 intersect;
		static CVector3 hitpos;
		static CVector3 l_hitpos_first; //store hitpos after RMB, for cube
		in_ctxt.GetMousePosition( x, y );
		//in_ctxt.Snap(x,y,50,siSnap0D | siSnap1D | siSnap2D,
		
		m_mouseDragX = x;
		m_mouseDragY = y;
		
		CVector3 l_offset, l_startpos;

		if (m_dragMode == kFirst && IsModKeyUp(in_ctxt) ) //entering secondary drag mode
		{
			if(m_paramSecCnt == 0)
				return CStatus::Abort;
			//SetCursorPos(lp_mouse.x,lp_mouse.y);	
			if(m_parameterDrawingAxisSec == kNormal)
				in_ctxt.SetCursor(siDragYCursor);
			//bring back cursor to original position
			m_mouseDownX = x;
			m_mouseDownY = y;
			
			m_dragMode = kSecond;
			l_hitpos_first = hitpos;
			in_ctxt.Redraw(false);
			
			return CStatus::OK;
		}
		else
			if (m_dragMode == kSecond && !IsModKeyUp(in_ctxt) ) // aborting mouse drag event
			{
				return CStatus::Abort;
			}

		double  l_doffset;	// OFFSET
		int		l_offsetsign = 1; // -/+ on axis
		CLine mray, l_mousedownray;
;	
		CPlane viewPlane, workPlane;
		in_ctxt.GetViewPlane(viewPlane);
		in_ctxt.SetManipulationPlane(viewPlane);
		in_ctxt.EnableSnapping(false);
		in_ctxt.GetWorldRay(m_mouseDownX,m_mouseDownY,l_mousedownray);
		in_ctxt.EnableSnapping(true);
		in_ctxt.GetWorldRay(x,y,mray);
		
		CPlane objecPlane(m_primitivePos,m_pickedFrameAxisY);
	
		CVector3 raynormal = l_mousedownray.GetTangent();
		raynormal.NormalizeInPlace();
		double anglePlane = objecPlane.GetNormal().Dot( raynormal );

		
		if (abs(anglePlane)> 0.015 )	
			workPlane = objecPlane;
		else 
			workPlane = viewPlane;
		if( m_primitive.GetType() == L"Null" ){//|| (abs(anglePlane)< 0.015 && m_dragMode != kSecond  )	){	//special case, null, or angle between picked normal vs viewplane is very small
			in_ctxt.GetWorldPosition(m_mouseDownX, m_mouseDownY, l_startpos);
			in_ctxt.GetWorldPosition(m_mouseDragX, m_mouseDragY, l_offset);
			
			l_offset -= l_startpos;
			l_doffset = abs(l_offset[0]+l_offset[1])*0.5;
		}
		else	
		{			
			if ( (m_dragMode == kFirst) || (m_parameterDrawingAxisSec == kPlane && m_dragMode == kSecond) ){	//drag along local reference frame
				//intersect mouse ray with objecPlane
				if(!workPlane.Intersect(mray,intersect))
					return CStatus::OK;

				double u,v;
		
				workPlane.GetParameters(intersect,u,v);
				
				hitpos = workPlane.GetPosition(u,v);
				
				CVector3 l_hitoffset( hitpos);
				l_hitoffset -= m_primitivePos;

				l_doffset = l_hitoffset.GetLength();

			}
			else	
				if(m_parameterDrawingAxisSec == kNormal)		//drag along normal (Y)
				{
					CValue is_snap;		
					Application().GetPreferences().GetPreferenceValue(L"SnapProperties.Enable",is_snap);
					
					CVector3 l_startpos, l_mousedownpos;
					in_ctxt.GetWorldPosition(m_mouseDownX,m_mouseDownY,l_mousedownpos);
					
					is_snap = (bool)is_snap != in_ctxt.IsControlKeyDown(); //preferences xor ctrl key
					
					CLine normal;
				
					if (is_snap)
						l_startpos = m_pickedSubcompPos;
					else
						l_startpos = l_mousedownpos;
					normal.Set(l_startpos,m_pickedFrameAxisY);
					if ( normal.Parallel( CLine( viewPlane.GetOrigin(), viewPlane.GetNormal() ) ) )	
						normal.Set(l_startpos,m_pickedFrameAxisZ);	
					
					CVector3 l_world;
					in_ctxt.GetWorldPosition(x,y,l_world);		
					CVector3 l_projectpt;
					if (is_snap)
						l_projectpt = l_world;
					else
						l_projectpt = normal.GetClosestPoint(mray);
					double l_param = normal.GetParameter( l_projectpt );
					l_offsetsign = l_param < 0 ? -1 : +1;
					CVector3 hitpoint = normal.GetPosition( normal.GetParameter( l_projectpt ) );
					hitpoint -= l_startpos;			
					l_doffset = hitpoint.GetLength()-0.01;
					
					if (m_primitiveType != L"Cone")
						l_doffset /= (2*m_heightFixRatio);
					 
					if (m_primitiveType == L"Cube")
							l_doffset *= 100.0;
					in_ctxt.ClearManipulationPlane();
				}
		}
		// Round to the nearest scale snap increment when control key pressed
		if ( in_ctxt.IsShiftKeyDown() )
		{
			double scaleSnap( 0.0 );
			CValue ret;
			if ( Application().GetPreferences().GetPreferenceValue(L"TransformProperties.scalesnap",ret) != CStatus::OK)
				scaleSnap = 0.5;
			else
				scaleSnap = ret;
			if (m_primitiveType == L"Cube")
				scaleSnap *= 100.0;
			l_doffset = GetClosestIncrement( l_doffset, scaleSnap );
		}

		if (l_doffset < 0.01) 	l_doffset = 0.01;
		
		if ( m_dragMode == kFirst )	{
			
			if(m_primitiveType == L"Grid" || m_primitiveType == L"Cube") //special case, drawing a grid!
			{	
				
				CLine a(m_primitivePos,m_pickedFrameAxisX);
				CLine b(m_primitivePos,m_pickedFrameAxisZ);
				
				CLine c(hitpos,m_pickedFrameAxisY);

				double offseta = a.GetParameter(a.GetClosestPoint(c));
				double offsetb = b.GetParameter(b.GetClosestPoint(c));
				CVector3 hita = a.GetPosition(offseta);
				CVector3 hitb = b.GetPosition(offsetb);
					
				hita-=m_primitivePos;
				hitb-=m_primitivePos;
				double pa=abs(hita.GetLength());
				double pb=abs(hitb.GetLength());
				if ( m_drawMode == kCenterOut)	{
					pa *= 2;
					pb *= 2;
				}
				// Round to the nearest scale snap increment when shift key pressed
				if ( in_ctxt.IsShiftKeyDown() )
				{
					double scaleSnap( 0.0 );
					CValue ret;
					if ( Application().GetPreferences().GetPreferenceValue(L"TransformProperties.scalesnap",ret) != CStatus::OK)
						scaleSnap = 0.5;
					else
						scaleSnap = ret;
					pa = GetClosestIncrement( pa, scaleSnap );
					pb = GetClosestIncrement( pb, scaleSnap );
				}

				if (pa < 0.01) pa= 0.01;
				if (pb < 0.01) pb= 0.01;
				if(m_piemenu.GetSelectionName() == L"Cube")	{
					pa*=100.0;
					pb*=100.0;
				}
					/*
				if(m_piemenu.GetSelectionName() == L"Cube")	{
						//uniform cube
					if (pb > pa)
						l_doffset = pb*2;
					else
						l_doffset = pa*2;
					in_ctxt.UpdateParameter(m_parameter[0], l_doffset/2);
				}
				else	*/
				
					in_ctxt.UpdateParameter(m_parameter[0], pa);
					in_ctxt.UpdateParameter(m_parameter[1], pb);			
			}
			else 
				if (m_paramCnt == 1)	{
					in_ctxt.UpdateParameter(m_parameter[0], l_doffset);
			}
		}
		else {
		
			if (m_paramSecCnt == 1 )	{
				CStatus l_stat = in_ctxt.UpdateParameter(m_parameterSec[0], l_doffset);
			}
			else 
				{		
					in_ctxt.UpdateParameter(m_parameterSec[0], abs(l_doffset));
					in_ctxt.UpdateParameter(m_parameterSec[1], abs(l_doffset));
				}
		}
		
		CTransformation l_primTransform(m_primitive.GetKinematics().GetGlobal().GetTransform());	
		l_primTransform.SetTranslation(CVector3(m_primitivePos) );
		double fix_y;
		if (m_paramSecCnt)
			fix_y = static_cast<double>( m_parameterSec[0].GetValue() );
		else
			fix_y = static_cast<double>( m_parameter[0].GetValue() );
		
		// drawing in negative direction (flip cone, move cylinder|cube
			
		fix_y *= m_heightFixRatio;
		if (l_offsetsign < 0 ){
			if (m_primitiveType == L"Cylinder" || m_primitiveType == L"Cube")			
				fix_y -= l_doffset;
			if (m_primitiveType == L"Cone")	{
				in_ctxt.UpdateParameter( m_primitive.GetKinematics().GetGlobal().GetParameter(L"scly"), (int)-1);
				fix_y = -fix_y;
			}
		}
		else 
			if (m_primitiveType == L"Cone")
				in_ctxt.UpdateParameter(m_primitive.GetKinematics().GetGlobal().GetParameter(L"scly"), (int)1);
		// multipler for cube as cube uses scaling not prim. parameters
		if (m_primitiveType == L"Cube")	
			fix_y *= 0.01;

		//special case: grid/cube drawing from point to point
		if ( (m_primitiveType == L"Grid" || m_primitiveType == L"Cube" ) && m_drawMode == kPointToPoint ) {
			double fixx = m_parameter[0].GetValue();
			double fixz;
		
			fixz= m_parameter[1].GetValue();

			CVector3 l_hit(hitpos);
			if (m_dragMode == kFirst)
				l_hit = hitpos;
			else
				l_hit = l_hitpos_first;
			l_hit -= m_primitivePos;
			fixx *=0.5; fixz *=0.5;
			if (m_primitiveType == L"Cube")	{
				fixx *=0.01;
				fixz *=0.01;
			}
						
			double anglex = l_hit.Dot(m_pickedFrameAxisX);
			if(anglex<0) 
				fixx=-fixx;
			double anglez = l_hit.Dot(m_pickedFrameAxisZ);
			if(anglez<0) 
				fixz=-fixz;;
			
			l_primTransform.AddLocalTranslation(CVector3(fixx,fix_y, fixz));
		}
		else
			l_primTransform.AddLocalTranslation(CVector3(0,fix_y,0));

		in_ctxt.UpdateParameter(m_primitive.GetKinematics().GetGlobal().GetParameter(L"posx"),l_primTransform.GetPosX());
		in_ctxt.UpdateParameter(m_primitive.GetKinematics().GetGlobal().GetParameter(L"posy"),l_primTransform.GetPosY());
		in_ctxt.UpdateParameter(m_primitive.GetKinematics().GetGlobal().GetParameter(L"posz"),l_primTransform.GetPosZ());
		
		return CStatus::OK;
	}
	// ******************************************************************************************************** CLEAR DATA

	void ClearPickedData()
	{		
		m_pickedSubcompType = kNone;
		m_dragMode = kFirst;
		m_toolMode = kIdle;		
		m_picked.ResetObject();
		m_pickedEdge = Edge();
		m_pickedPolyFace = PolygonFace();
		m_pickedVertex = Vertex();
		m_pickedSubcompString = L"";
		m_activeAxis = kNoDragAxis;
	}
	void ClearData()
	{

		ClearPickedData();
		m_primitive.ResetObject();
		m_parameter[0] = Parameter();
		m_parameter[1] = Parameter();
		m_parameter[2] = Parameter();
		m_parameterSec[0] = Parameter();
		m_parameterSec[1] = Parameter();
		m_parameterSec[2] = Parameter();
		m_paramCnt		= 0;
		m_paramSecCnt	= 0;

		m_cprimitives.Clear();
	}
	void ClearAllData()
	{	
		ClearPickedData();
		ClearData();	
	}
	//************** save/get data
	CStatus GetSavedData(CString in_varname, LONG &out_val)
	{
		CValue retval;
		CValueArray args(1);
		args[0] = in_varname;
		Application().ExecuteCommand( L"GetGlobal", args, retval );
		out_val = (LONG)retval;
#ifdef _DEBUG
		logmsg("Loading data:"+ in_varname, out_val );
#endif
		return retval.IsEmpty() ? CStatus::Fail : CStatus::OK;
	}	

	void SaveData( LONG in_value,CString in_varname )
	{
#ifdef _DEBUG
		logmsg("Saving data:"+ in_varname, in_value );
#endif
		CValue retval;
		CValueArray args(2);
		args[0] = in_varname;
		args[1] = in_value;
		Application().ExecuteCommand( L"SetGlobal", args, retval );
	}
	// ******************************************************************************************************************************************* GET SNAP DATA
	X3DObject GetComponent(ToolContext& in_ctxt,CRef &target)
	{			
			X3DObject l_meshObject;
			CComAPIHandler cCollItem(target);
			CComAPIHandler cSubComp(cCollItem.GetProperty(L"SubComponent"));
			CComAPIHandler cCompColl(cSubComp.GetProperty(L"ComponentCollection"));
			CValueArray indices = cSubComp.GetProperty( L"ElementArray" );
			l_meshObject = cSubComp.GetProperty( L"Parent3DObject" ); 
			
			PolygonMesh pmesh( l_meshObject.GetActivePrimitive().GetGeometry() );
			CString l_subcomp = cSubComp.GetProperty(L"Type");
			if(LONG(indices.GetCount() ) == 1L)
				if ( l_subcomp == L"edgeSubComponent" ) {
					m_pickedEdge = pmesh.GetEdges().GetItem((LONG)indices[0]);	
					m_pickedSubcompType = kEdge;
				}
				else if ( l_subcomp == L"polySubComponent" ) {
					m_pickedPolyFace = pmesh.GetPolygons().GetItem((LONG)indices[0]);
					m_pickedSubcompType = kPoly;
				}
				else if ( l_subcomp == L"pntSubComponent" ) {
						m_pickedVertex = pmesh.GetVertices().GetItem((LONG)indices[0]); 
						m_pickedSubcompType = kVertex;	
					};	
			return l_meshObject;
	}
	CStatus GetGeometrySubcomponentData(ToolContext& in_ctxt, LONG x, LONG y, CRefArray& in_objects, CVector3& out_pickedPos)
	{
		CLongArray l_points;
		l_points.Add(x);
		l_points.Add(y);
		CRefArray components;
		X3DObject l_picked = in_objects[0];
		
		in_ctxt.Pick( l_points, siPickSingleSubComponent, siPickRaycast, siPointFilter, in_objects, components );		
		if( components.GetCount() == 0)	{					
			in_ctxt.Pick( l_points, siPickSingleSubComponent, siPickRaycast, siEdgeFilter, in_objects, components );
			if( components.GetCount() == 0)
				in_ctxt.Pick( l_points, siPickSingleSubComponent, siPickRaycast, siPolygonFilter, in_objects, components );	
		}

		if( components.GetCount() != 0 )
		{
			m_picked	 = GetComponent(in_ctxt,components[0]);
			CTransformation localToGlobal = m_picked.GetKinematics().GetGlobal().GetTransform();

			//get position on comopnent
			if(m_pickedSubcompType == kVertex) {		//for point, it's just position of the vertex
				out_pickedPos = m_pickedVertex.GetPosition();
				out_pickedPos.MulByTransformationInPlace(localToGlobal);
			}
			else
				if(m_pickedSubcompType == kEdge)			//for edge it's intersection of cursor ray with edge vector
				{
					CLine l_cursorRay;
					in_ctxt.GetWorldRay( x, y, l_cursorRay );
					CVertexRefArray vx = m_pickedEdge.GetNeighborVertices();
					Vertex	v( vx[0] ),	v2( vx[1] );
					CVector3 pt1 = v.GetPosition();
					CVector3 pt2 = v2.GetPosition();
								
					pt1.MulByTransformationInPlace(localToGlobal);
					pt2.MulByTransformationInPlace(localToGlobal);
					pt2.SubInPlace(pt1);
					CLine l_edgeLine(pt1,pt2);
					out_pickedPos = l_edgeLine.GetClosestPoint( l_cursorRay );
				}
				else
					if(m_pickedSubcompType == kPoly)		//for poly its raycasting cursor ray onto facet
					{
						CLine l_cursorRay;
						in_ctxt.EnableSnapping(false);
						in_ctxt.GetWorldRay( x, y, l_cursorRay );
						in_ctxt.EnableSnapping(true);
						PolygonMesh pmesh( m_picked.GetActivePrimitive().GetGeometry() );
						const LONG ind[1] = {m_pickedPolyFace.GetIndex()};
									
						if ( pmesh.SetupPointLocatorQueries(siClosestSurfaceRaycastIntersection,&localToGlobal,1,ind,-1) != CStatus::OK)
							return CStatus::False;
						CVector3 rayt=l_cursorRay.GetTangent();
						CVector3 rayp=l_cursorRay.GetOrigin();
						PointLocatorData closestPointLocator = pmesh.GetRaycastIntersections( 1, (double*)&rayp, (double*)&rayt, siLineIntersection  );
						if (!closestPointLocator.IsPtLocatorValid(0) )
							return CStatus::False;
					
						double pos[3];
						if (pmesh.EvaluatePositions(closestPointLocator, -1, 0, pos) != CStatus::OK)
							return CStatus::False;
						
						out_pickedPos.Set(pos[0],pos[1],pos[2]);
						out_pickedPos.MulByTransformationInPlace(localToGlobal);									
					}	
			switch (m_pickedSubcompType){
				case kEdge:		m_pickedNormal = getEdgeNormal(in_ctxt,m_pickedEdge, false);		break;
				case kPoly:		m_pickedNormal = getPolyNormal(in_ctxt,m_pickedPolyFace, false);	break;
				case kVertex:	m_pickedNormal = getVertexNormal(in_ctxt,m_pickedVertex, false);	break;
				default:break;
			}										
		}	//end if components picked
		else
			return CStatus::False;
		return CStatus::OK;
	}

	CRefArray ExpandSelection( ToolContext& in_ctxt, CRefArray sel)		//convert subcomponent selection to objects
	{	
		CRefArray	l_objects; 		
		for (int i=0;i < sel.GetCount();i++)
		{		
			X3DObject obj;
			if (!sel[i].IsA( siX3DObjectID ) )
			{
				CComAPIHandler comCollItem( sel.GetItem(i) );
				CComAPIHandler comSubComp( comCollItem.GetProperty(L"SubComponent") );		
				obj = comSubComp.GetProperty( L"Parent3DObject" );
			}
			else 
				obj = sel[i];
			if (obj.IsA( siX3DObjectID ) ) 					
				l_objects.Add(obj);					
		}
		return l_objects;
	}

	CStatus GetSnapData(ToolContext& in_ctxt, LONG x, LONG y, CVector3 &l_primPos)
	{
		ClearPickedData();

		CRefArray	selection = Application().GetSelection().GetArray();
		CStringArray l_snapFamily;

		
		CLongArray l_points;
		l_points.Add( x );
		l_points.Add( y );
		CRefArray objects;
	
		if ( m_toolscopeButtons.GetSelectionIndex() ){	//not grid scope
			CRefArray in_objects;
			 if ( m_toolscopeButtons.GetSelectionIndex() == 1 ){
				 
				 in_objects = ExpandSelection(in_ctxt, selection);
				 
			 }
			in_ctxt.Pick( l_points, siPickSingleObject, siPickRaycast,L"", in_objects, objects );
		}
		CStatus pickStatus=CStatus::False;
		if ( objects.GetCount() > 0 ) 
		{ 	
			CString objclass = objects[0].GetClassIDName();
			if (objclass== L"X3DObject")
			{
				X3DObject l_picked = objects[0];
				CString l_pickedType = l_picked.GetType();
				
				if (l_pickedType == L"polymsh")		{	
					pickStatus = GetGeometrySubcomponentData(in_ctxt,x,y, objects,l_primPos);
					if( pickStatus != CStatus::OK	)
						return pickStatus;
				}
				else
					if (l_pickedType == L"surfmsh"){
						NurbsSurfaceMesh nmesh(l_picked.GetActivePrimitive().GetGeometry());
						CLine l_cursorRay;
						in_ctxt.GetWorldRay( x, y, l_cursorRay );
						
						const LONG ind[1] = {-1};
						CTransformation localToGlobal = l_picked.GetKinematics().GetGlobal().GetTransform();			
						nmesh.SetupPointLocatorQueries(siClosestSurfaceRaycastIntersection,&localToGlobal,-1,ind,-1);
						CVector3 rayt=l_cursorRay.GetTangent();
						CVector3 rayp=l_cursorRay.GetOrigin();

						PointLocatorData closestPointLocator = nmesh.GetRaycastIntersections( 2, (double*)&rayp, (double*)&rayt,  siSegmentIntersection  );
						if (closestPointLocator.IsValid())
						{
							double pos[6];
							nmesh.EvaluatePositions(closestPointLocator,-1,0, pos);	
						
							LONG l_si;
							double l_di,l_uv,l_vv;
							CVector3 l_outpos(pos[0],pos[1],pos[2]);
							
							pickStatus = nmesh.GetClosestSurfacePosition(l_outpos,l_si,l_di,l_uv,l_vv,l_outpos);
							if( pickStatus == CStatus::OK)
							{
							
								NurbsSurface surf = nmesh.GetSurfaces().GetItem(l_si);
								pickStatus = surf.EvaluatePosition(l_uv,l_vv,l_outpos,m_pickedFrameAxisX,m_pickedFrameAxisZ,m_pickedFrameAxisY);
								if( pickStatus == CStatus::OK)
								{
									l_primPos = l_outpos;
									l_primPos.MulByTransformationInPlace(localToGlobal);	
						
									m_pickedCurveFrameAxisX = m_pickedFrameAxisX;
									m_pickedCurveFrameAxisY = m_pickedFrameAxisY;
									m_pickedCurveFrameAxisZ = m_pickedFrameAxisZ;
									m_pickedCurveFrameAxisX.Cross( m_pickedCurveFrameAxisY, m_pickedCurveFrameAxisZ ); 
									m_picked = l_picked;
								}						
							}
							else
								return pickStatus;
						}
						else
							return CStatus::Fail;
					}
					else
						if(l_pickedType ==L"crvlist")
						{
							CStringArray snapfamilies;
						
							in_ctxt.EnableSnapping(true);
							pickStatus = in_ctxt.Snap(x,y,70,siSnapCurve | siSnapKnot,snapfamilies,objects,l_primPos);
							if (pickStatus == CStatus::OK) {
								siSnapType snaptype = in_ctxt.GetSnapType();

								if (snaptype == siSnapCurve || snaptype == siSnapKnot)
								{
							
									NurbsCurveList l_curvelist( l_picked.GetActivePrimitive().GetGeometry() );
									LONG l_crvIndex; double l_dist; double l_uvalue;CVector3 l_outpos;

									CTransformation localToGlobal = l_picked.GetKinematics().GetGlobal().GetTransform();
									CTransformation inv;
									MATH::MapObjectPoseToWorldSpace(localToGlobal,inv);							
									CVector3 l_pose = l_primPos;
									l_pose.MulByTransformationInPlace(inv);	//move snapped position to object space
									l_curvelist.GetClosestCurvePosition(l_pose,l_crvIndex,l_dist,l_uvalue,l_outpos);
									CNurbsCurveRefArray l_curves(l_curvelist.GetCurves());
									NurbsCurve l_curve( l_curves.GetItem(l_crvIndex) );
									pickStatus = l_curve.EvaluatePosition(l_uvalue,l_outpos,m_pickedFrameAxisX,m_pickedFrameAxisY,m_pickedFrameAxisZ);
									m_pickedCurveFrameAxisX = m_pickedFrameAxisX;
									m_pickedCurveFrameAxisY = m_pickedFrameAxisY;
									m_pickedCurveFrameAxisZ = m_pickedFrameAxisZ;
									m_pickedCurveFrameAxisX.Cross( m_pickedCurveFrameAxisY, m_pickedCurveFrameAxisZ ); 
									m_picked = l_picked;
								}			
							}
							//in_ctxt.EnableSnapping(false);
						}
						
			}		//if x3dobject
			else
				if(objclass== L"Null")	{
					m_picked = objects[0];
					l_primPos = m_picked.GetKinematics().GetGlobal().GetTransform().GetTranslation();	
					pickStatus = CStatus::OK;
				}
				
		}	// end	 if ( objects.GetCount() > 0 ) 
		
		bool forceGridAlign=false;
		//we picked subcomponent but aligning to this subcomp. is disabled - force grid align
		if (m_pickedSubcompType != kNone)
			forceGridAlign = !m_alignButtons.IsSelected( m_pickedSubcompType  );
		
		//grid align
		if( (!objects.GetCount() || ( ( objects.GetCount() || selection.GetCount() ) && (pickStatus != CStatus::OK) ) ) || ( forceGridAlign ) )	{			//no objects picked OR object picked but component not picked, get normal from grid plane

			if ( !( m_toolscopeButtons.GetSelectionIndex() == 1 && selection.GetCount() )	|| forceGridAlign )	{

				CPlane gridPlane;
				in_ctxt.GetGridPlane(gridPlane);	

				m_pickedNormal = gridPlane.GetNormal();
				in_ctxt.SetManipulationPlane(gridPlane);
				if (!forceGridAlign)
					pickStatus = in_ctxt.GetWorldPosition(x,y,l_primPos) ;
				in_ctxt.ClearManipulationPlane();
				// Reject intersection points behind the camera
				CVector3 cameraPoint;
				if ( in_ctxt.WorldToCamera( l_primPos, cameraPoint ) != CStatus::OK || cameraPoint.GetZ() > 0.0 )
					pickStatus = CStatus::Fail;
				//set position on grid only if not forcing aligning
				if (!forceGridAlign)
					m_pickedSubcompPos = l_primPos;

				m_pickedFrameAxisX = gridPlane.GetTangentU();
				m_pickedFrameAxisY = gridPlane.GetNormal();
				m_pickedFrameAxisZ = gridPlane.GetTangentV();
				m_pickedFrameAxisZ.NormalizeInPlace();
				m_pickedFrameAxisZ.NegateInPlace();
				m_pickedFrameAxisX.Cross(m_pickedFrameAxisY,m_pickedFrameAxisZ);
				m_pickedFrameAxisX.NormalizeInPlace();
			}
		}

		return pickStatus;
	}
	// ******************************************************************************************************** BEGIN PARAM UPDATE
	void calcPrimData(CString str, CStringArray &l_param,CStringArray &l_paramSec)
	{
		m_heightFixRatio = 0.5;
		m_parameterDrawingAxisSec = kPlane;
		if (str == L"Sphere")	{
			l_param.Add(L"radius");
			m_paramCnt = 1;
			m_paramSecCnt = 0;
			m_heightFixRatio = 1;
		}
		else
			if (str == L"Cube")	{
				
				/*
				l_param.Add(L"length");
				m_paramCnt = 1;
				m_paramSecCnt = 0;*/
				l_param.Add(L"sclx");
				l_param.Add(L"sclz");
				l_paramSec.Add(L"scly");
				m_paramCnt = 2;
				m_paramSecCnt = 1;
				m_parameterDrawingAxisSec = kNormal;
			}
			else
				if (str == L"Grid")	{
					l_param.Add("ulength");
					l_param.Add("vlength");
					m_paramSecCnt = 0;
					m_paramCnt = 2;
					m_heightFixRatio=0;
				}
				else
					if (str == L"Cylinder")	{
						l_param.Add(L"radius");
						l_paramSec.Add(L"height");
						m_paramSecCnt = 1;
						m_paramCnt = 1;
						m_parameterDrawingAxisSec = kNormal;
					}
						else
						if (str == L"Torus")	{
							l_param.Add(L"radius");
							l_paramSec.Add(L"sectionradius");
							m_paramSecCnt = 1;
							m_paramCnt = 1;
							m_heightFixRatio = 1;
							m_parameterDrawingAxisSec = kNormal;
						}
						else
							if (str == L"Disc")	{
								l_param.Add(L"outerradius");
								l_paramSec.Add(L"innerradius");
								m_paramSecCnt = 1;
								m_paramCnt = 1;
								m_heightFixRatio = 0;
							}
							else
								if (str == L"Dodecahedron")	{
									l_param.Add(L"radius");
								
									m_paramSecCnt = 0;
									m_paramCnt = 1;
									m_heightFixRatio = 0.79465;
								}
								else
									if (str == L"Cone")	{
										l_param.Add(L"radius");
										l_paramSec.Add(L"height");
										m_paramSecCnt = 1;
										m_paramCnt = 1;
										m_heightFixRatio = 1.0/3.0;
										m_parameterDrawingAxisSec = kNormal;
									}
									else
										if (str == L"Circle")	{
											l_param.Add(L"radius");
											m_paramSecCnt = 0;
											m_paramCnt = 1;
											m_heightFixRatio = 0;
										}
										else
										if (str == L"Null")	{
											l_param.Add(L"size");
											m_paramSecCnt = 0;
											m_paramCnt = 1;
											m_heightFixRatio = 0;
										}

	}
	
	//save data for MMB duplication
	void SavePrimtiveData(CString str, X3DObject l_primitive)
	{
		m_lastprimitiveData.Clear();
		if (str == L"Sphere")	
			m_lastprimitiveData.Add(l_primitive.GetParameterValue(L"radius"));
		else
			if (str == L"Cube")	{
				m_lastprimitiveData.Add(l_primitive.GetParameterValue(L"sclx"));
				m_lastprimitiveData.Add(l_primitive.GetParameterValue(L"sclz"));
				m_lastprimitiveData.Add(l_primitive.GetParameterValue(L"scly"));
				m_lastprimitiveData.Add(l_primitive.GetParameterValue(L"length"));
			}
			else
				if (str == L"Grid")	{
					m_lastprimitiveData.Add(l_primitive.GetParameterValue("ulength"));
					m_lastprimitiveData.Add(l_primitive.GetParameterValue("vlength"));
				}
				else
					if (str == L"Cylinder")	{
						m_lastprimitiveData.Add(l_primitive.GetParameterValue(L"radius"));
						m_lastprimitiveData.Add(l_primitive.GetParameterValue(L"height"));
					}
						else
						if (str == L"Torus")	{
							m_lastprimitiveData.Add(l_primitive.GetParameterValue(L"radius"));
							m_lastprimitiveData.Add(l_primitive.GetParameterValue(L"sectionradius"));
						}
						else
							if (str == L"Disc")	{
								m_lastprimitiveData.Add(l_primitive.GetParameterValue(L"outerradius"));
								m_lastprimitiveData.Add(l_primitive.GetParameterValue(L"innerradius"));
							}
							else
								if (str == L"Dodecahedron")	
									m_lastprimitiveData.Add(l_primitive.GetParameterValue(L"radius"));
								else
									if (str == L"Cone")	{
										m_lastprimitiveData.Add(l_primitive.GetParameterValue(L"radius"));
										m_lastprimitiveData.Add(l_primitive.GetParameterValue(L"height"));
									}
									else
										if (str == L"Circle")
											m_lastprimitiveData.Add(l_primitive.GetParameterValue(L"radius"));
										else
										if (str == L"Null")	
											m_lastprimitiveData.Add(l_primitive.GetParameterValue(L"size"));
										
		if ( (str != L"Null" && str != L"Dodecahedron") )
				m_lastprimitiveData.Add( l_primitive.GetParameterValue(L"subdivu"));
		if (str != L"Circle" && str != L"Null" && str != L"Dodecahedron") 
				m_lastprimitiveData.Add( l_primitive.GetParameterValue(L"subdivv"));
		if  (str != L"Grid" && str != L"Sphere" && str != L"Disc" && str != L"Torus" &&	str != L"Circle" && str != L"Null" && m_primitiveType != L"Dodecahedron" ) 
				m_lastprimitiveData.Add( l_primitive.GetParameterValue(L"subdivbase" ));
	}

	//load data for duplication
	double LoadPrimtiveData(CString str, X3DObject &out_primtive)
	{		
		LONG i = 0;
		double height = 0;
		if (str == L"Sphere")	{
			height = m_lastprimitiveData[i];
			out_primtive.PutParameterValue(L"radius",m_lastprimitiveData[i++]);
		}
		else
			if (str == L"Cube")	{				
				out_primtive.PutParameterValue(L"sclx",m_lastprimitiveData[i++]); 
				out_primtive.PutParameterValue(L"sclz",m_lastprimitiveData[i++]); 			
				out_primtive.PutParameterValue(L"scly",m_lastprimitiveData[i++]); 
				height = static_cast<double>( m_lastprimitiveData[i]) * static_cast<double>( m_lastprimitiveData[ 2 ] );
				out_primtive.PutParameterValue(L"length",m_lastprimitiveData[i++]); 
				
			}
			else
				if (str == L"Grid")	{
					out_primtive.PutParameterValue(L"ulength",m_lastprimitiveData[i++]);
					out_primtive.PutParameterValue(L"vlength",m_lastprimitiveData[i++]);
				}
				else
					if (str == L"Cylinder")	{
						out_primtive.PutParameterValue(L"radius",m_lastprimitiveData[i++]);
						height = m_lastprimitiveData[i];
						out_primtive.PutParameterValue(L"height",m_lastprimitiveData[i++]);
					}
						else
						if (str == L"Torus")	{						
							out_primtive.PutParameterValue(L"radius",m_lastprimitiveData[i++]);
							height = m_lastprimitiveData[i];
							out_primtive.PutParameterValue(L"sectionradius",m_lastprimitiveData[i++]);
						}
						else
							if (str == L"Disc")	{
								out_primtive.PutParameterValue(L"outerradius",m_lastprimitiveData[i++]);
								out_primtive.PutParameterValue(L"innerradius",m_lastprimitiveData[i++]);
							}
							else
								if (str == L"Dodecahedron")	{
									height = m_lastprimitiveData[i];
									out_primtive.PutParameterValue(L"radius",m_lastprimitiveData[i++]);
								}
								else
									if (str == L"Cone")	{
										out_primtive.PutParameterValue(L"radius",m_lastprimitiveData[i++]);
										height = m_lastprimitiveData[i];
										out_primtive.PutParameterValue(L"height",m_lastprimitiveData[i++]);
									}
									else
										if (str == L"Circle")
											out_primtive.PutParameterValue(L"radius",m_lastprimitiveData[i++]);
										else
											if (str == L"Null")	
												out_primtive.PutParameterValue(L"size",m_lastprimitiveData[i++]);

		if ( (str != L"Null" && str != L"Dodecahedron") )
			out_primtive.PutParameterValue(L"subdivu",m_lastprimitiveData[i++]);
		if (str != L"Circle" && str != L"Null" && str != L"Dodecahedron") 
			out_primtive.PutParameterValue(L"subdivv", m_lastprimitiveData[i++]) ;
		if  (str != L"Grid" && str != L"Sphere" && str != L"Disc" && str != L"Torus" &&	str != L"Circle" && str != L"Null" && m_primitiveType != L"Dodecahedron" ) 
					out_primtive.PutParameterValue(L"subdivbase",m_lastprimitiveData[i++]);
		return height;
	}

	CStatus BeginParamUpdate(ToolContext& in_ctxt, X3DObject &in_prim)
	{
		
		in_ctxt.BeginParameterUpdate( in_prim.GetKinematics().GetGlobal().GetParameter(L"posx") );
		in_ctxt.BeginParameterUpdate( in_prim.GetKinematics().GetGlobal().GetParameter(L"posy") );
		in_ctxt.BeginParameterUpdate( in_prim.GetKinematics().GetGlobal().GetParameter(L"posz") );
		if (m_primitiveType == L"Cone")
			in_ctxt.BeginParameterUpdate( in_prim.GetKinematics().GetGlobal().GetParameter(L"scly") );
		CString str = m_piemenu.GetSelectionName();
		CStringArray l_param, l_paramSec;	
		calcPrimData(str,l_param,l_paramSec);
			
		CParameterRefArray l_params = m_primitive.GetParameters();
		if (m_primitiveType == L"Cube")
			in_prim.PutParameterValue(L"length",0.01);
		
		for (int i=0;i<m_paramCnt;i++) {
			m_parameter[i] = l_params.GetItem(l_param[i]);
			in_ctxt.BeginParameterUpdate( m_parameter[i]);
			if (m_primitiveType != L"Cube") 
				in_ctxt.UpdateParameter(m_parameter[i], 0.01);
		}
		for (int i=0;i<m_paramSecCnt;i++) {
			m_parameterSec[i] = l_params.GetItem(l_paramSec[i]);
			in_ctxt.BeginParameterUpdate( m_parameterSec[i]);
			if (m_primitiveType != L"Cube") 
				in_ctxt.UpdateParameter(m_parameterSec[i], 0.01);
		}

		//push object in Y dir. (because we can't create primitive scaled to 0
		CTransformation lTransform = in_prim.GetKinematics().GetGlobal().GetTransform();	
		double fix_y;
		if (m_paramSecCnt)
			fix_y = static_cast<double>( m_parameterSec[0].GetValue() );
		else
			fix_y = static_cast<double>( m_parameter[0].GetValue() );
		
		if (m_primitiveType == L"Cube")
			fix_y = 0.01;
		fix_y *= m_heightFixRatio;
		lTransform=lTransform.AddLocalTranslation(CVector3(0,fix_y,0));
	
		in_prim.GetKinematics().GetGlobal().PutTransform(lTransform);
		return CStatus::OK;
	}

	// ******************************************************************************************************** END PARAM UPDATE
	
	CStatus EndParamUpdate(ToolContext& in_ctxt, X3DObject &in_prim)
	{
#ifdef _DEBUG
		logmsg(L"Ending parameter update");
#endif
		in_ctxt.EndParameterUpdate( in_prim.GetKinematics().GetGlobal().GetParameter(L"posx") );
		in_ctxt.EndParameterUpdate( in_prim.GetKinematics().GetGlobal().GetParameter(L"posy") );
		in_ctxt.EndParameterUpdate( in_prim.GetKinematics().GetGlobal().GetParameter(L"posz") );
		if (m_primitiveType == L"Cone")
			in_ctxt.EndParameterUpdate( in_prim.GetKinematics().GetGlobal().GetParameter(L"scly") );	//cone uses scaling for direction
		for (int i=0;i<m_paramCnt;i++) 
			in_ctxt.EndParameterUpdate(  m_parameter[i] );
		for (int i=0;i<m_paramSecCnt;i++) 
			in_ctxt.EndParameterUpdate(  m_parameterSec[i] );
		return CStatus::OK;
	}
	
	CRotation AlignPrimitiveToComponentNormal(ToolContext& in_ctxt, double& out_rotx,double& out_roty,double& out_rotz)
	{
		// align primitive rotation to normal, taken from SI ICE compound 'increment rotation with 2 vectors' ...

		CTransformation l_pickedTransform(m_picked.GetKinematics().GetGlobal().GetTransform());
		CQuaternion rotq = l_pickedTransform.GetRotationQuaternion();
		CVector3 normal=m_pickedNormal;
		CVector3 up(0,1,0);
		CVector3 cross;
		cross.Cross(up,normal);
		CRotation rot;	
		double dot= up.Dot(normal);
		dot++;
		
		if (dot !=0)
		{
			dot*=2;
			dot=sqrt(dot);
			cross*= (1/dot);

			dot /=2.0;
			CQuaternion outq(dot,cross[0],cross[1],cross[2]);

			rot = outq.GetRotation();
		}
		else
			rot.SetFromAxisAngle(CVector3(1,0,0),DegreesToRadians(180));

		rotq.MulInPlace(rot);

		rot.SetFromQuaternion(rotq);
		rot.GetXYZAngles(out_rotx,out_roty,out_rotz);
		
		out_rotx=RadiansToDegrees(out_rotx);
		out_roty=RadiansToDegrees(out_roty);
		out_rotz=RadiansToDegrees(out_rotz);
		return rot;
	}
	void GetPointRefFrame(ToolContext& in_ctxt,X3DObject& in_obj, Vertex& in_vert, CVector3& outX, CVector3& outY, CVector3& outZ)
	{
		//ICE VERSION: (slow)
		/*
		ICEAttribute l_refframe = m_picked.GetActivePrimitive().GetGeometry().GetICEAttributeFromName(L"PointReferenceFrame");

		CICEAttributeDataArrayMatrix3f l_v;
		l_refframe.GetDataArray(l_v); 

		CMatrix3f cm(l_v[in_vert.GetIndex()]);

		outX.Set(cm.GetValue(0,0),cm.GetValue(0,1),cm.GetValue(0,2));
		outY.Set(cm.GetValue(1,0),cm.GetValue(1,1),cm.GetValue(1,2));
		outZ.Set(cm.GetValue(2,0),cm.GetValue(2,1),cm.GetValue(2,2));
		*/

		// fast version
		// coded after DOC's local frame magic formula...
			
		bool boolvalid;

		outY=in_vert.GetNormal(boolvalid);
		//calc edge vector
		CEdgeRefArray eref = in_vert.GetNeighborEdges();
		
		Edge e(eref[0]);
		CVector3Array ep = e.GetPoints().GetPositionArray();
		CVector3 edgevector = ep[1];
		edgevector-=ep[0];
		edgevector.NormalizeInPlace();

		// outx = calc normalized projection of edgevector onto (outx) normal plane
		double d=edgevector.Dot(outY);
		CVector3 n=outY;
		n*=d;
		outX=edgevector;
		outX-=n;
		outX.NormalizeInPlace();
		outX.NegateInPlace();
		// z is cross of x,y
		outZ.Cross(outX,outY);	
	
	}

	void GetEdgeRefFrame(ToolContext& in_ctxt,X3DObject& in_obj, Edge& in_edge, CVector3& outX, CVector3& outY, CVector3& outZ)
	{
		outY=getEdgeNormal(in_ctxt,m_pickedEdge,false);
		CVector3Array ep = in_edge.GetVertices().GetPositionArray();
		outX = ep[1];
		outX-= ep[0];
		outX.NormalizeInPlace();
		outY.NormalizeInPlace();
		outZ.Cross(outX,outY);	
		outZ.NormalizeInPlace();	
	}
	
	void GetPolyRefFrame(ToolContext& in_ctxt, X3DObject& in_obj, PolygonFace& in_poly, CVector3& outX, CVector3& outY, CVector3& outZ)
	{	
		outY = getPolyNormal(in_ctxt,m_pickedPolyFace,false);
		
		CVector3Array ep = in_poly.GetVertices().GetPositionArray();
		CVector3 edgevector = ep[0];
		CVector3 edgesub=ep[1];
		edgesub-=edgevector;
		outX = edgesub;
		outX.NormalizeInPlace();	
		outZ.Cross(outX,outY);	
		outZ.NormalizeInPlace();
	}
	CVector3 AlignPrimitiveToComponentRefFrame(ToolContext& in_ctxt, X3DObject& in_obj, CVector3& out_rotx,CVector3& out_roty,CVector3& out_rotz, bool switchaxes)
	{
		out_rotx.PutNull();
		out_roty.PutNull();
		out_rotz.PutNull();
		
		if(m_pickedSubcompType == kVertex)
			GetPointRefFrame(in_ctxt,in_obj,m_pickedVertex,out_rotx,out_roty,out_rotz);
		else 
			if(m_pickedSubcompType == kEdge)
				GetEdgeRefFrame(in_ctxt,in_obj,m_pickedEdge,out_rotx,out_roty,out_rotz);
			else
				if(m_pickedSubcompType == kPoly)
					GetPolyRefFrame(in_ctxt,in_obj,m_pickedPolyFace,out_rotx,out_roty,out_rotz);
		
		if (switchaxes)
			if(m_axisButtons.GetSelectionIndex() == 2 )
			{
				CVector3 temp = out_roty;
				out_roty = out_rotz;
				out_rotz = temp;
			}
			else
				if(m_axisButtons.GetSelectionIndex() == 0 )
				{
					CVector3 temp = out_roty;
					out_roty = out_rotx;
					out_rotx = temp;
				}
		double rx,ry,rz;
		
		CTransformation ct=in_obj.GetKinematics().GetGlobal().GetTransform();
		ct.SetPosX(0);ct.SetPosY(0);ct.SetPosZ(0);//reset position
		double sx,sy,sz;
		ct.GetScalingValues(sx,sy,sz);
		sx = sx < 0 ? -1 : 1;
		sy = sy < 0 ? -1 : 1;
		sz = sz < 0 ? -1 : 1;
		ct.SetScaling(CVector3(sx,sy,sz));
		out_rotx.MulByTransformationInPlace(ct);
		out_roty.MulByTransformationInPlace(ct);
		out_rotz.MulByTransformationInPlace(ct);
		out_rotx.NormalizeInPlace();
		out_roty.NormalizeInPlace();
		out_rotz.NormalizeInPlace();
		//make sure vectors are orthogonal
		out_rotx.Cross( out_roty, out_rotz ); 
		
        out_rotx.NormalizeInPlace(); 
		 
		CRotation rot;	
		rot.SetFromXYZAxes(out_rotx,out_roty,out_rotz);
		
		rot.GetXYZAngles(rx,ry,rz);

		return CVector3(RadiansToDegrees(rx),RadiansToDegrees(ry),RadiansToDegrees(rz));
	}

	//read subdiv levels from preferences and update prim
	void SetPrimitiveSubdivisions(CString str,X3DObject &in_obj)	{
		int pcnt = 0;
		int i,j,k;
		i=j=k=1;

		if (str == L"Sphere" || str == L"Grid" || str == L"Disc" || str == L"Torus")	{
			pcnt = 2;
			GetFromPrefs(str+L"U", i);
			GetFromPrefs(str+L"V", j);		
		}
		else
			if (str == L"Cylinder"  || str == L"Cone" || str == L"Cube")	{
				pcnt = 3;
				GetFromPrefs(str+L"U", i);
				GetFromPrefs(str+L"V", j);
				GetFromPrefs(str+L"Base", k);
			}
		if(pcnt){
			in_obj.GetParameter(L"subdivu").PutValue(i);
			in_obj.GetParameter(L"subdivv").PutValue(j);
		}
		if(pcnt==3)
			in_obj.GetParameter(L"subdivbase").PutValue(k);
		
	}

	CStatus CreatePrimitive(ToolContext& in_ctxt, CString l_primstr, CVector3& l_primPos, X3DObject &out_obj)
	{
		Model l_root = Application().GetActiveSceneRoot();
		m_primitiveType = l_primstr;
		
		CString l_primType = L"MeshSurface";		
		if(l_primstr == L"Null")
			l_primType = L"";
		else 
			if(l_primstr == L"Circle")
				l_primType = "NurbsCurve";

		CStatus l_primcreateStatus=l_root.AddGeometry(l_primstr, l_primType, l_primstr, out_obj);

		if (l_primcreateStatus != CStatus::OK) {
			logmsg("Could not create primitive!");
			return l_primcreateStatus;
		}

		SetPrimitiveSubdivisions(l_primstr,out_obj);

		CRotation l_rot;
		CVector3 axex,axey,axez;	//output local ref frame axes

		if (m_alignButtons.IsSelected(m_pickedSubcompType)  )
		{

			if (m_alignMode == kNormalAlign)	{	//not tested
				double x,y,z;
				l_rot = AlignPrimitiveToComponentNormal(in_ctxt,x,y,z);
				//l_rot.SetFromXYZAngles(x,y,z);
			}
			else {
				AlignPrimitiveToComponentRefFrame(in_ctxt,m_picked,axex,axey,axez,true);
				l_rot.SetFromXYZAxes(axex,axey,axez);
			}	
		}
		else
			if (m_picked.GetType() == L"null")
			{
				CTransformation transf= m_picked.GetKinematics().GetGlobal().GetTransform();	
				CVector3 t= transf.GetTranslation();
				CMatrix3 m = transf.GetRotationMatrix3();
				switch (m_axisButtons.GetSelectionIndex() )	{
				case 0:
					axex = CVector3(0,1,0);
					axey = CVector3(1,0,0);
					axez = CVector3(0,0,1);
					break;
				case 2:
					axex = CVector3(1,0,0);
					axey = CVector3(0,0,1);
					axez = CVector3(0,1,0);
					break;
				default:
					axex = CVector3(1,0,0);
					axey = CVector3(0,1,0);
					axez = CVector3(0,0,1);
					break;
				}
				axex.MulByMatrix3InPlace(m);axey.MulByMatrix3InPlace(m);	axez.MulByMatrix3InPlace(m);
				axex.Cross( axey, axez ); 
				l_rot.SetFromXYZAxes(axex,axey,axez);
			}
			else
				//if (m_picked.GetType() == L"crvlist" || m_picked.GetType() == L"surfmsh")
				{
					switch (m_axisButtons.GetSelectionIndex() )	{
					case 0:
						axey = m_pickedFrameAxisX;
						axex = m_pickedFrameAxisY;
						axez = m_pickedFrameAxisZ;
						break;
					case 2:
						axex = m_pickedFrameAxisX;
						axez = m_pickedFrameAxisY;
						axey = m_pickedFrameAxisZ;
						break;
					default:
						axex = m_pickedFrameAxisX;
						axey = m_pickedFrameAxisY;
						axez = m_pickedFrameAxisZ;
						break;
					}
					axex.Cross( axey, axez );
					l_rot.SetFromXYZAxes(axex,axey,axez);
				}
		
			//	else 
				//{			// create prim aligned to grid plane
					/*
					switch (m_axisButtons.GetSelectionIndex() )	{
					case 0:
						axex = CVector3(0,1,0);
						axey = CVector3(1,0,0);
						axez = CVector3(0,0,1);
						break;
					case 2:
						axex = CVector3(1,0,0);
						axey = CVector3(0,0,1);
						axez = CVector3(0,1,0);
						break;
					default:
						axex = CVector3(1,0,0);
						axey = CVector3(0,1,0);
						axez = CVector3(0,0,1);
						break;
					}*/
			/*
					axex = m_pickedFrameAxisX;
					axey = m_pickedFrameAxisY;
					axez = m_pickedFrameAxisZ;
					//axex.Cross(axey,axez);
					axex.NormalizeInPlace();
					axey.NormalizeInPlace();
					axez.NormalizeInPlace();
					l_rot.SetFromXYZAxes(axex,axey,axez);
				}	
				**/
		CTransformation lTransform = out_obj.GetKinematics().GetGlobal().GetTransform();
		if (l_primstr == L"Circle")	{		//circle,rotate it
			CRotation l_crot(DegreesToRadians(-90),0,0);
			lTransform.SetRotation(l_crot);
			out_obj.GetKinematics().GetGlobal().PutTransform(lTransform);
			//freeze rotation
			CValue retval;
			CValueArray args(4);
			args[0] = out_obj.GetFullName();
			args[1] = L"siCtr";
			args[2] = L"siRot";
			args[3] = L"siXYZ";
			
			Application().ExecuteCommand( L"ResetTransform", args, retval );	//freeze rotation
		}

		lTransform.SetTranslation(l_primPos);
		lTransform.SetRotation(l_rot);
		out_obj.GetKinematics().GetGlobal().PutTransform(lTransform);

		m_pickedFrameAxisX = axex;
		m_pickedFrameAxisY = axey;
		m_pickedFrameAxisZ = axez;
		return CStatus::OK;
	}

	void RescaleNormalVector(CVector3 &in_startv, CVector3 io_v,double scale)
	{	
		CVector3 vsub;
		vsub.Sub(io_v,in_startv);
		vsub.NormalizeInPlace();
		vsub.ScaleInPlace(scale);
		vsub.AddInPlace(in_startv);
		io_v=vsub;
	}
	CVector3 getEdgeNormal(ToolContext& in_ctxt,Edge &in_edge,bool in_mulmatrix)
	{
		
		CPolygonFaceRefArray l_ep(in_edge.GetNeighborPolygons());
		int cnt = l_ep.GetCount();
		CVector3 vn;
		for (int i=0;i<cnt;i++)
		{
			PolygonFace face(l_ep[i]);
			CVector3 s,e,n;
			n=getPolyNormal(in_ctxt,face,false);
			n.NormalizeInPlace();
			vn+=n;
		}
		X3DObject in_obj=in_edge.GetParent();
		if(in_mulmatrix)
			vn*=in_obj.GetKinematics().GetGlobal().GetTransform().GetRotationMatrix3();
		vn.NormalizeInPlace();
		return vn;
	}
	CVector3 getVertexNormal(ToolContext& in_ctxt,Vertex &in_v, bool in_mulmatrix)		//returns normal
	{
		CPolygonFaceRefArray l_vp(in_v.GetNeighborPolygons());
		int cnt = l_vp.GetCount();
		CVector3 vn;

		for (int i=0;i<cnt;i++)
		{
			PolygonFace face(l_vp[i]);
			CVector3 s,e,n;
			n=getPolyNormal(in_ctxt, face,false);
			vn+=n;
		}
		
		X3DObject	in_obj=in_v.GetParent();
		if(in_mulmatrix)
			vn*=in_obj.GetKinematics().GetGlobal().GetTransform().GetRotationMatrix3();
		vn.NormalizeInPlace();
		return vn;
	}
	// calculate polygon normal (Newell's method) http://www.opengl.org/wiki/Calculating_a_Surface_Normal
	CVector3 getPolyNormal(ToolContext& in_ctxt, PolygonFace &in_poly, bool in_mulmatrix)
	{
			X3DObject			in_obj=in_poly.GetParent();
			PolygonMesh			mesh(in_obj.GetActivePrimitive().GetGeometry() );
			CVertexRefArray		verts(in_poly.GetVertices());
			CLongArray			tri_ind(in_poly.GetTriangleSubIndexArray());
			CVector3Array		n,p;	
							
			UINT vertCnt = verts.GetCount();
			double x=0,y=0,z=0;
			for(UINT i=0;i<vertCnt;i++)
			{
				Vertex current = verts[i];			
				Vertex next = verts[ (i+1) % vertCnt];
				CVector3 c(current.GetPosition() );
				CVector3 n(next.GetPosition() );
				x= x+ ( (c[1]-n[1])* (c[2]+n[2]) );
				y= y+ ( (c[2]-n[2])* (c[0]+n[0]) );
				z= z+ ( (c[0]-n[0])* (c[1]+n[1]) );
			}

			CVector3 normal(x,y,z);		
			if(in_mulmatrix)
				normal*=in_obj.GetKinematics().GetGlobal().GetTransform().GetRotationMatrix3();
			normal.NormalizeInPlace();
			return normal;
	}

	LONG GetClosestDragAxis( ToolContext &in_ctxt, const CVector3 &in_mouseVec, CLine in_axes[], double &out_projection ) const
	{
		// Calculate axis that best matches the mouse drag direction in screen-space
		LONG result = -1; // No axis
		double maxProjection = kHandleDragThreshold;
		for ( int i = 0; i < 3; ++i )
		{
			CVector3 initial, axis;
			if ( in_ctxt.WorldToView( in_axes[i].GetOrigin(), initial ) != CStatus::OK ||
				 in_ctxt.WorldToView( in_axes[i].GetPosition(1.0), axis ) != CStatus::OK ){
				return result;
			}
			// Convert to a 2D vector (without depth)
			axis.SubInPlace( initial ).PutZ( 0.0 );
			double lenSquared = axis.GetLengthSquared();
			if ( lenSquared > kHandleDragThreshold*kHandleDragThreshold )
			{
				double projectedLen = fabs(in_mouseVec.Dot(axis)) * axis.GetLength() / lenSquared;
				if ( projectedLen > maxProjection )
				{
					result = i;
					out_projection = maxProjection = projectedLen;
				}
			}
		}
		return result;
	}

	CValue SelectObject( X3DObject &in_obj, bool in_bBranch )
	{
		CValue result;
		CValueArray args;
		args.Add( in_obj.GetFullName() );
		args.Add( in_bBranch ? L"BRANCH" : L"NODE");
		Application().ExecuteCommand( L"SelectObj", args, result );
		return result;
	}
	CStatus KeyUp( ToolContext& in_ctxt )
	{	
		/*
		switch ( (ULONG) in_ctxt.GetShortcutKey() ) 		{
			
			default:
			break;
		}*/
		return CStatus::OK;
	}

	CStatus KeyDown( ToolContext& in_ctxt )
	{
		CString l_paramstr;
		CValueArray args(1);
		CValue outArg;		
		X3DObject l_obj;
		LONG l_val;
		Parameter l_param;				
		l_obj = m_primitive;
		bool l_updateparam = false;

		switch ( (ULONG) in_ctxt.GetShortcutKey() ) 
		{
			case 0x0D: //enter, inspect last op
				if (m_lastPrimType == kLight)
					l_obj = m_lastLightprimitive;
				if ( l_obj.IsValid() ) {
					args[0] = l_obj.GetFullName();
					Application().ExecuteCommand( L"InspectObj", args, outArg );
			}
			break;
		case kSubdiv1:
				if (m_primitiveType == L"Null" || m_primitiveType == L"Dodecahedron")
					return CStatus::OK;
				l_paramstr = L"subdivu";
				l_updateparam = true;
			break;
		case kSubdiv2:
				if (m_primitiveType == L"Circle" || m_primitiveType == L"Null" || m_primitiveType == L"Dodecahedron")
					return CStatus::OK;
				l_paramstr = L"subdivv";
				l_updateparam = true;
			break;
		case kSubdiv3:
				if (m_primitiveType == L"Grid" || m_primitiveType == L"Sphere" || m_primitiveType == L"Disc" || m_primitiveType == L"Torus" ||
					m_primitiveType == L"Circle" || m_primitiveType == L"Null" || m_primitiveType == L"Dodecahedron" )
					return CStatus::OK;
				l_paramstr = L"subdivbase";
				l_updateparam = true;
			break;
			default:
			break;
		}

		if ( l_updateparam )	{
			
			l_param = l_obj.GetParameter( l_paramstr );
			if ( l_param.IsValid() )	{
				l_val = l_param.GetValue();
				if(in_ctxt.GetShortcutKeyModifiers() == 1)
					l_val--;
				else
					l_val++;
				if(l_val < 1) l_val = 1;
				l_param.PutValue(l_val);
			
				SavePrimtiveData(m_primitiveType, l_obj);

				ClearPickedData();
			}
		}
		return CStatus::OK;
	}
}; // end class

////////////////////////////////////////////////////////////////////////////////
//
// Plugin Callbacks
//
////////////////////////////////////////////////////////////////////////////////
SICALLBACK pmQuickPrimitive_Menu_Init( CRef& in_ctxt )
{
	Context ctxt( in_ctxt );
 	MenuItem menuItem;
 	Menu menu = ctxt.GetSource();
 	menu.AddCommandItem( L"Quick Primitive", L"pmQuickPrimitive", menuItem );
 	return true;
}

SICALLBACK pmQuickPrimitive_Init( CRef& in_ctxt )
{
	pmQuickPrimitive *l_pTool = new pmQuickPrimitive;
	if ( !l_pTool ) return CStatus::Fail;
	ToolContext l_ctxt( in_ctxt );
	l_ctxt.PutUserData( CValue((CValue::siPtrType)l_pTool) );
	return CStatus::OK;
}

SICALLBACK pmQuickPrimitive_Term( CRef& in_ctxt )
{
	ToolContext l_ctxt( in_ctxt );
	pmQuickPrimitive *l_pTool = (pmQuickPrimitive *)(CValue::siPtrType)l_ctxt.GetUserData();
	if ( !l_pTool ) return CStatus::Fail;
	delete l_pTool;
	l_ctxt.PutUserData( CValue((CValue::siPtrType)NULL) ); // Clear user data
	return CStatus::OK;
}


// Use a macro to simplify callback setup and forwarding...

#define DECLARE_TOOL_CALLBACK(TOOL,CALLBACK) \
SICALLBACK TOOL##_##CALLBACK( ToolContext& in_ctxt ) { \
	TOOL *l_pTool = (TOOL *)(CValue::siPtrType)in_ctxt.GetUserData(); \
	return ( l_pTool ? l_pTool->CALLBACK( in_ctxt ) : CStatus::Fail ); }

DECLARE_TOOL_CALLBACK( pmQuickPrimitive, Setup );
DECLARE_TOOL_CALLBACK( pmQuickPrimitive, Cleanup );
DECLARE_TOOL_CALLBACK( pmQuickPrimitive, Activate );
DECLARE_TOOL_CALLBACK( pmQuickPrimitive, Deactivate );
DECLARE_TOOL_CALLBACK( pmQuickPrimitive, MouseDown );
DECLARE_TOOL_CALLBACK( pmQuickPrimitive, MouseDrag );
DECLARE_TOOL_CALLBACK( pmQuickPrimitive, MouseUp );
DECLARE_TOOL_CALLBACK( pmQuickPrimitive, Draw );
DECLARE_TOOL_CALLBACK( pmQuickPrimitive, MouseMove );
DECLARE_TOOL_CALLBACK( pmQuickPrimitive, MouseEnter );
DECLARE_TOOL_CALLBACK( pmQuickPrimitive, MouseLeave );
DECLARE_TOOL_CALLBACK( pmQuickPrimitive, SnapValid );
DECLARE_TOOL_CALLBACK( pmQuickPrimitive, KeyUp );
DECLARE_TOOL_CALLBACK( pmQuickPrimitive, KeyDown );

